# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from osc_lib import exceptions

from otcextensions.osclient.rds.v3 import backup
from otcextensions.tests.unit.osclient.rds.v3 import fakes as fakes


class TestList(fakes.TestRds):

    _instance = fakes.FakeInstance.create_one()

    _objects = fakes.FakeBackup.create_multiple(3)

    column_headers = ('ID', 'Name', 'Type', 'Instance Id', 'Size (KB)',
                      'Begin time', 'End time')
    columns = ('id', 'name', 'type', 'instance_id', 'size',
               'begin_time', 'end_time')

    data = []

    for s in _objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestList, self).setUp()

        self.cmd = backup.ListBackup(self.app, None)

        self.client.backups = mock.Mock()
        self.client.find_instance = mock.Mock(return_value=self._instance)

    def test_list_default(self):
        arglist = ['test-instance']

        verifylist = [
            ('instance', 'test-instance'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.backups.side_effect = [self._objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('test-instance',
                                                     ignore_missing=False)
        self.client.backups.assert_called_once_with(instance=self._instance)

        self.assertEqual(self.column_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_params(self):
        arglist = [
            'test-instance',
            '--backup-id', 'bid',
            '--backup-type', 'manual',
            '--offset', '1',
            '--limit', '2',
            '--begin-time', '3',
            '--end-time', '4'
        ]

        verifylist = [
            ('instance', 'test-instance'),
            ('backup_id', 'bid'),
            ('backup_type', 'manual'),
            ('offset', 1),
            ('limit', 2),
            ('begin_time', '3'),
            ('end_time', '4')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.backups.side_effect = [self._objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with(
            'test-instance',
            ignore_missing=False)
        self.client.backups.assert_called_once_with(
            instance=self._instance,
            backup_id='bid',
            backup_type='manual',
            offset=1,
            limit=2,
            paginated=False,
            begin_time='3',
            end_time='4'
        )

        self.assertEqual(self.column_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_wrong_filter(self):
        arglist = [
            'test-instance',
            '--begin-time', 'asd'
        ]

        verifylist = [
            ('instance', 'test-instance'),
            ('begin_time', 'asd')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args
        )


class TestCreate(fakes.TestRds):

    _data = fakes.FakeBackup.create_one()
    _instance = fakes.FakeInstance.create_one()

    columns = (
        'begin_time', 'datastore', 'description',
        'end_time', 'id', 'instance_id', 'name',
        'size', 'status', 'type'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreate, self).setUp()

        self.cmd = backup.CreateBackup(self.app, None)

        self.client.create_backup = mock.Mock(return_value=self._data)
        self.client.find_instance = mock.Mock(return_value=self._instance)
        self.client.wait_for_backup = mock.Mock(return_value=self._data)

    def test_create(self):
        arglist = [
            'test-backup',
            'test-instance',
            '--description',
            'test description',
        ]
        verifylist = [
            ('name', 'test-backup'),
            ('instance', 'test-instance'),
            ('description', 'test description'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('test-instance',
                                                     ignore_missing=False)
        self.client.create_backup.assert_called_with(
            instance=self._instance,
            description='test description',
            name='test-backup'
        )
        self.assertEqual(self.columns, columns)

    def test_create_wait(self):
        arglist = [
            'test-backup',
            'test-instance',
            '--description',
            'test description',
            '--wait', '--wait-interval', '5'
        ]
        verifylist = [
            ('name', 'test-backup'),
            ('instance', 'test-instance'),
            ('description', 'test description'),
            ('wait', True),
            ('wait_interval', 5)
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('test-instance',
                                                     ignore_missing=False)
        self.client.create_backup.assert_called_with(
            instance=self._instance,
            description='test description',
            name='test-backup'
        )
        self.client.wait_for_backup.assert_called_with(
            self._data,
            interval=5
        )
        self.assertEqual(self.columns, columns)

    def test_create_sqlserver(self):
        arglist = [
            'test-backup',
            'test-instance',
            '--description', 'test description',
            '--databases', 'a,b,c'
        ]
        verifylist = [
            ('name', 'test-backup'),
            ('instance', 'test-instance'),
            ('description', 'test description'),
            ('databases', 'a,b,c')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self._instance.datastore.update({'type': 'sqlserver'})
        self.client.find_instance.return_value = self._instance

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_instance.assert_called_with('test-instance',
                                                     ignore_missing=False)
        self.client.create_backup.assert_called_with(
            instance=self._instance,
            description='test description',
            name='test-backup',
            databases=[{'name': 'a'}, {'name': 'b'}, {'name': 'c'}]
        )
        self.assertEqual(self.columns, columns)


class TestDelete(fakes.TestRds):
    def setUp(self):
        super(TestDelete, self).setUp()

        self.cmd = backup.DeleteBackup(self.app, None)

        self.client.delete_backup = mock.Mock()

    def test_delete_default(self):
        arglist = [
            'bck1',
        ]

        verifylist = [
            ('backup', ['bck1']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_backup.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_backup.assert_called_once_with(backup='bck1',
                                                          ignore_missing=False)

    def test_delete_multiple(self):
        arglist = ['bck1', 'bck2']

        verifylist = [
            ('backup', ['bck1', 'bck2']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_backup.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [
            mock.call(backup='bck1', ignore_missing=False),
            mock.call(backup='bck2', ignore_missing=False),
        ]

        self.client.delete_backup.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_backup.call_count)


class TestListLinks(fakes.TestRds):

    _objects = fakes.FakeBackupFile.create_multiple(3)

    column_headers = ('Size (KB)', 'URL', 'Expires at')
    columns = ('size', 'download_link', 'expires_at')

    data = []

    for s in _objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListLinks, self).setUp()

        self.cmd = backup.ListBackupDownloadLinks(self.app, None)

        self.client.backup_download_links = mock.Mock(
            return_value=self._objects)

    def test_list_default(self):
        arglist = ['test-backup']

        verifylist = [
            ('backup_id', 'test-backup'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.backup_download_links.assert_called_once_with(
            backup_id='test-backup')

        self.assertEqual(self.column_headers, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.elb.v2 import _proxy
from otcextensions.sdk.elb.v2 import listener_tag
from otcextensions.sdk.elb.v2 import load_balancer_tag


class TestElbProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestElbProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)
        self.session.get_project_id = mock.Mock(return_value='aa')


class TestElbLoadBalancerTag(TestElbProxy):
    def test_load_balancer_tag_create(self):
        self.verify_create(
            self.proxy.create_load_balancer_tag,
            load_balancer_tag.Tag,
            method_kwargs={
                'key': 'key1',
                'value': 'value1',
                'load_balancer': 'id',
            },
            expected_kwargs={
                'key': 'key1',
                'value': 'value1',
                'loadbalancer_id': 'id',
            }
        )

    def test_load_balancer_delete(self):
        self.verify_delete(
            self.proxy.delete_load_balancer_tag,
            load_balancer_tag.Tag,
            True,
            method_kwargs={
                'key': 'resource_id',
            },
            expected_kwargs={
                'loadbalancer_id': 'resource_id',
                'ignore_missing': True
            }
        )

    def test_load_balancers(self):
        self.verify_list(
            self.proxy.load_balancer_tags,
            load_balancer_tag.Tag,
            method_kwargs={
                'load_balancer': 'id',
            },
            expected_kwargs={
                'loadbalancer_id': 'id',
            }
        )


class TestElbLoadBalancerListenerTag(TestElbProxy):
    def test_listener_tag_create(self):
        self.verify_create(
            self.proxy.create_listener_tag,
            listener_tag.Tag,
            method_kwargs={
                'key': 'key1',
                'value': 'value1',
                'listener': 'id',
            },
            expected_kwargs={
                'key': 'key1',
                'value': 'value1',
                'listener_id': 'id',
            }
        )

    def test_load_balancer_delete(self):
        self.verify_delete(
            self.proxy.delete_listener_tag,
            listener_tag.Tag,
            True,
            method_kwargs={
                'key': 'resource_id',
            },
            expected_kwargs={
                'listener_id': 'resource_id',
                'ignore_missing': True
            }
        )

    def test_listeners(self):
        self.verify_list(
            self.proxy.listener_tags,
            listener_tag.Tag,
            method_kwargs={
                'listener': 'id',
            },
            expected_kwargs={
                'listener_id': 'id',
            }
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CBR Restore point CLI implementation'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_checkpoint(obj):
    """Flatten the structure of the checkpoint into a single dict
    """
    vault_id, vault_name, backup_name = ('', '', '')
    if obj.vault:
        vault_id = obj.vault.id
        vault_name = obj.vault.name
    if obj.extra_info:
        backup_name = obj.extra_info.name
    data = {
        'created_at': obj.created_at,
        'id': obj.id,
        'project_id': obj.project_id,
        'status': obj.status,
        'vault_id': vault_id,
        'vault_name': vault_name,
        'backup_name': backup_name
    }

    return data


def _normalize_resources(resource_details):
    result = []
    for rd in resource_details:
        res = dict(map(lambda s: s.split('='), rd.split(' ')))
        result.append(res)
    return result


def _add_resources_to_obj(obj, data, columns):
    """Add resources to obj.vault
    """
    i = 0
    for s in obj.vault.resources:
        if obj.vault.resources[i].id:
            name = 'resource_' + str(i + 1)
            data += (obj.vault.resources[i].id,)
            columns = columns + (name,)
            i += 1
    return data, columns


def _add_skipped_resources_to_obj(obj, data, columns):
    """Add skipped resources to obj.vault
    """
    i = 0
    for s in obj.vault.skipped_resources:
        if obj.vault.resources[i].id:
            name = 'skipped_resource_' + str(i + 1)
            data += (obj.vault.skipped_resources[i].id,)
            columns = columns + (name,)
            i += 1
    return data, columns


class ShowCheckpoint(command.ShowOne):
    _description = _('Show single restore point details')
    columns = (
        'id',
        'status',
        'created_at',
        'vault_id',
        'vault_name',
        'backup_name'
    )

    def get_parser(self, prog_name):
        parser = super(ShowCheckpoint, self).get_parser(prog_name)
        parser.add_argument(
            'checkpoint',
            metavar='<checkpoint>',
            help=_('ID of the CBR checkpoint.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        obj = client.get_checkpoint(
            checkpoint=parsed_args.checkpoint
        )

        data = utils.get_dict_properties(
            _flatten_checkpoint(obj), self.columns)

        return (self.columns, data)


class CreateCheckpoint(command.ShowOne):
    _description = _('Create CBR Restore point')
    columns = (
        'id',
        'status',
        'created_at',
        'vault_id',
        'vault_name',
        'backup_name'
    )

    def get_parser(self, prog_name):
        parser = super(CreateCheckpoint, self).get_parser(prog_name)

        parser.add_argument(
            '--vault-id',
            metavar='<vault_id>',
            required=True,
            help=_('Vault id.')
        )
        parser.add_argument(
            '--auto-trigger',
            dest='auto_trigger',
            action='store_true',
            help=_('Whether automatic triggering is enabled.')
        )
        parser.add_argument(
            '--description',
            default='backup',
            help=_('Backup description.')
        )
        parser.add_argument(
            '--no-incremental',
            dest='no_incremental',
            action='store_false',
            help=_('Scheduling rule.\n'
                   'Repeat Option for multiple rules.')
        )
        parser.add_argument(
            '--backup-name',
            metavar='backup_name',
            help=_('Backup name.')
        )
        parser.add_argument(
            '--resources',
            action='append',
            help=_('UUID list of resources to be backed up.')
        )

        parser.add_argument(
            '--resource-details',
            metavar='<resource_details>',
            action='append',
            help=_('Associated resource in "id=resource_id '
                   'type=resource_type name=resource_name" format.'
                   'Repeat for multiple values.')
        )

        return parser

    def take_action(self, parsed_args):

        attrs = {
            'parameters': {},
        }

        # mandatory
        attrs['vault_id'] = parsed_args.vault_id

        # optional

        if parsed_args.auto_trigger is not None:
            attrs['parameters'].update(auto_trigger=parsed_args.auto_trigger)
        if parsed_args.description:
            attrs['parameters'].update(
                description=parsed_args.description)
        if parsed_args.no_incremental is not None:
            attrs['parameters'].update(incremental=parsed_args.no_incremental)
        if parsed_args.backup_name:
            attrs['parameters'].update(
                name=parsed_args.backup_name)
        if parsed_args.resources:
            attrs['parameters'].update(
                resources=parsed_args.resources)
        if parsed_args.resource_details:
            attrs['parameters'].update(resource_details=_normalize_resources(
                parsed_args.resource_details))

        client = self.app.client_manager.cbr
        obj = client.create_checkpoint(**attrs)

        data = utils.get_dict_properties(
            _flatten_checkpoint(obj), self.columns)

        if obj.vault.resources:
            data, self.columns = _add_resources_to_obj(obj, data, self.columns)

        if obj.vault.skipped_resources:
            data, self.columns = _add_skipped_resources_to_obj(obj, data,
                                                               self.columns)

        return (self.columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import exceptions


class ApigTriggerFuncInfo(resource.Resource):
    # Function URN
    function_urn = resource.Body('function_urn', type=str)
    # Execution mode of a function.
    # sync: synchronous execution
    # async: asynchronous execution
    invocation_type = resource.Body('invocation_type', type=str)
    # Timeout allowed for APIG to request the FunctionGraph service.
    # The unit is in millisecond. This parameter is mandatory
    # for APIG triggers.
    timeout = resource.Body('timeout', type=int)
    # Function version information.
    version = resource.Body('version', type=str)


class TriggerEventData(resource.Resource):
    # Timer trigger: trigger name
    # APIG trigger: API name
    # CTS trigger: Notification name
    # OBS trigger: Event Notification name.
    # The default value is The trigger ID.
    name = resource.Body('name', type=str)
    # Timer trigger type (timer trigger parameter).
    # This parameter is mandatory for timer triggers.
    # Rate: specifies the frequency (minutes, hours, or days)
    # at which the function is invoked. If the unit is minute,
    # the value cannot exceed 60. If the unit is hour,
    # the value cannot exceed 24. If the unit is day,
    # the value cannot exceed 30.
    # Cron: specifies a Cron expression to periodically invoke a function.
    schedule_type = resource.Body('schedule_type', type=str)
    # Triggering rule. (timer trigger parameter).
    # This parameter is mandatory for timer triggers.
    schedule = resource.Body('schedule', type=str)
    # Additional information (timer trigger parameter).
    # When the timer trigger triggers the function,
    # the execution event (the event parameter of the function)
    # is {"version": "v1.0", "time": "2018-06-01T08:30:00+08:00",
    # "trigger_type": "TIMER", "trigger_name": "Timer_001",
    # "user_event": "Additional information you entered"}.
    user_event = resource.Body('user_event', type=str)
    # APIG trigger ID (APIG trigger parameter).
    apig_trigger_id = resource.Body('triggerid', type=str)
    # API visibility(APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    # 1: public
    # 2: private
    type = resource.Body('type', type=int)
    # Path of the API (APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    path = resource.Body('path', type=str)
    # Request protocol of the API (APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    # Enumeration values:
    # HTTP
    # HTTPS
    protocol = resource.Body('protocol', type=str)
    # Request method of the API (APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    # Enumeration values:
    # GET
    # POST
    # PUT
    # DELETE
    # HEAD
    # PATCH
    # OPTIONS
    # ANY
    req_method = resource.Body('req_method', type=str)
    # ID of the group to which the API belongs (APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    group_id = resource.Body('group_id', type=str)
    # Name of the API group to which the API belongs.
    group_name = resource.Body('group_name', type=str)
    # Matching mode of the API (APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    # SWA: prefix match
    # NORMAL: normal match (absolute match)
    match_mode = resource.Body('match_mode', type=str)
    # Environment in which the API is published (APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    env_name = resource.Body('env_name', type=str)
    # ID of the environment in which the API has been published
    # (APIG trigger parameter) This parameter is mandatory for APIG triggers.
    env_id = resource.Body('env_id', type=str)
    # API ID (APIG trigger parameter).
    api_id = resource.Body('api_id', type=str)
    # Security authentication (APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    # IAM: IAM authentication.
    # Only IAM users are allowed to access the system.
    # The security level is medium.
    # APP: Appkey&Appsecret authentication is used.
    # The security level is high.
    # This authentication mode is recommended.
    # NONE: No authentication mode is used.
    # All users can access the system. This mode is not recommended.
    auth = resource.Body('auth', type=str)
    # FunctionGraph backend details (APIG trigger parameter).
    # This parameter is mandatory for APIG triggers.
    func_info = resource.Body('func_info', type=ApigTriggerFuncInfo)
    # API calling address (APIG trigger parameter).
    invoke_url = resource.Body('invoke_url', type=str)
    # Subdomain name allocated by the APIG
    # system by default (APIG trigger parameter).
    sl_domain = resource.Body('sl_domain', type=str)
    # Backend type of the API (APIG trigger parameter).
    # Enumeration values:
    # FUNCTION
    backend_type = resource.Body('backend_type', type=str)
    # Custom operations (CTS trigger parameter).
    # This parameter is mandatory for CTS triggers.
    # CTS collects operation records of subscribed cloud resources.
    # If you create a function with a CTS trigger,
    # collected operation records of specified cloud services
    # will be passed as a parameter to invoke the function.
    operations = resource.Body('operations', type=list)
    # Instance ID. This parameter is mandatory for DDS,
    # Kafka, and RabbitMQ triggers.
    # APIG trigger: APIG gateway ID
    # DDS trigger: DB instance ID.
    # Kafka trigger: Kafka instance ID
    # RabbitMQ trigger: RabbitMQ instance ID
    instance_id = resource.Body('instance_id', type=str)
    # ID of the integration application to which the API belongs
    # (APIG trigger parameter).
    roma_app_id = resource.Body('roma_app_id', type=str)
    # Collection name (DDS trigger parameter).
    # This parameter is mandatory for DDS triggers.
    collection_name = resource.Body('collection_name', type=str)
    # Database name (DDS trigger parameter).
    # This parameter is mandatory for DDS triggers.
    db_name = resource.Body('db_name', type=str)
    # DDS database password (DDS trigger parameter).
    # This parameter is mandatory for DDS triggers.
    db_password = resource.Body('db_password', type=str)
    # DDS database username (DDS trigger parameter).
    db_user = resource.Body('db_user', type=str)
    # DDS database instance address (DDS trigger parameter).
    instance_addrs = resource.Body('instance_addrs', type=list)
    # DDS database instance type (DDS trigger parameter).
    # Sharding: cluster instance
    # ReplicaSet: replica set instance
    # Single: single node instance.
    mode = resource.Body('mode', type=str)
    # Batch size: Maximum number of data records that can be processed by the
    # function at a time. This parameter is mandatory for DIS,
    # DDS, Kafka, and RabbitMQ triggers.
    # DDS trigger: Set the batch size to a value ranging from 1 to 10,000.
    # DIS trigger: Set the batch size to a value ranging from 1 to 10,000.
    # Kafka trigger: Set the batch size to a value ranging from 1 to 1,000.
    # RabbitMQ trigger: Set the batch size to a value ranging from 1 to 1,000.
    batch_size = resource.Body('batch_size', type=int)
    # Queue ID (DMS trigger parameter).
    # This parameter is mandatory for DMS triggers.
    queue_id = resource.Body('queue_id', type=str)
    # Consumer group ID (DMS trigger parameter).
    # This parameter is mandatory for DMS triggers.
    consumer_group_id = resource.Body('consumer_group_id', type=str)
    # Pull period. This parameter is mandatory for DIS and DMS triggers.
    polling_interval = resource.Body('polling_interval', type=int)
    # Stream name (DIS trigger parameter).
    # This parameter is mandatory for DIS triggers.
    stream_name = resource.Body('stream_name', type=str)
    # Starting position (DIS trigger parameter).
    # This parameter is mandatory for DIS triggers.
    # TRIM_HORIZON: Data is read from the earliest valid
    # record stored in the partition.
    # LATEST: Data is read from the latest record in the partition.
    # This option ensures that the most recent data in the partition is read.
    sharditerator_type = resource.Body('sharditerator_type', type=str)
    # Pull period unit (DIS trigger parameter).
    # This parameter is mandatory for DIS triggers.
    # s: second
    # ms: millisecond
    polling_unit = resource.Body('polling_unit', type=str)
    # Max. fetch bytes (DIS trigger parameter).
    max_fetch_bytes = resource.Body('max_fetch_bytes', type=int)
    # Serial data processing (DIS trigger parameter).
    # If enabled, FunctionGraph pulls data from the stream only
    # after previous data is processed. If disabled,
    # FunctionGraph pulls data from the stream as long
    # as the pull period ends. This parameter is mandatory for DIS triggers.
    # Enumeration values:
    # true
    # false
    is_serial = resource.Body('is_serial', type=str)
    # Log group ID (LTS trigger parameter).
    # This parameter is mandatory for LTS triggers.
    log_group_id = resource.Body('log_group_id', type=str)
    # Log stream ID (LTS trigger parameter).
    # This parameter is mandatory for LTS triggers.
    log_topic_id = resource.Body('log_topic_id', type=str)
    # Bucket name (OBS trigger parameter).
    # The name of the OBS bucket used as the event source
    # cannot be the same as that of an existing bucket
    # of the current user or another user. After being created,
    # the bucket name cannot be modified.
    # This parameter is mandatory for OBS triggers.
    bucket = resource.Body('bucket', type=str)
    # Prefix (OBS trigger parameter).
    # Enter a prefix to limit notifications to objects
    # whose names start with the matching characters.
    prefix = resource.Body('prefix', type=str)
    # Suffix (OBS trigger parameter).
    # Enter a suffix to limit notifications to objects
    # whose names end with the matching characters.
    suffix = resource.Body('suffix', type=str)
    # Trigger event (OBS trigger parameter).
    # This parameter is mandatory for OBS triggers.
    # ObjectCreated: all object creation operations,
    # including PUT, POST, COPY, and part assembling
    # Put: Uploads an object using Put.
    # Post: Uploads an object using POST
    # Copy: Copies an object using COPY.
    # CompleteMultipartUpload: Merges parts of a multipart upload.
    # ObjectRemoved: Deletes an object.
    # Delete: Deletes an object by specifying its version ID.
    # DeleteMarkerCreated: Deletes an object without specifying its version ID.
    events = resource.Body('events', type=list)
    # Topic URN (SMN trigger parameter).
    # This parameter is mandatory for SMN triggers.
    topic_urn = resource.Body('topic_urn', type=str)
    # The Kafka topic ID list (Kafka trigger parameter).
    # This parameter is mandatory for Kafka triggers.
    topic_ids = resource.Body('topic_ids', type=list)
    # Kafka account name (Kafka trigger parameter).
    kafka_user = resource.Body('kafka_user', type=str)
    # Kafka password (Kafka trigger parameter).
    kafka_password = resource.Body('kafka_password', type=str)
    # Kafka instance connection address (Kafka trigger parameter).
    kafka_connect_address = resource.Body('kafka_connect_address', type=str)
    # Whether to enable SASL authentication(Kafka trigger parameter).
    kafka_ssl_enable = resource.Body('kafka_ssl_enable', type=bool)
    # RabbitMQ account password (RabbitMQ trigger parameter).
    # This parameter is mandatory for RabbitMQ triggers.
    access_password = resource.Body('access_password', type=str)
    # RabbitMQ username (RabbitMQ trigger parameter).
    access_user = resource.Body('access_user', type=str)
    # Instance IP address (RabbitMQ trigger parameter).
    connect_address = resource.Body('connect_address', type=str)
    # Switch name (RabbitMQ trigger parameter).
    # This parameter is mandatory for RabbitMQ triggers.
    exchange_name = resource.Body('exchange_name', type=str)
    # Virtual host (RabbitMQ trigger parameter).
    vhost = resource.Body('vhost', type=str)
    # Whether to enable SSL (RabbitMQ trigger parameter).
    ssl_enable = resource.Body('ssl_enable', type=bool)


class Trigger(resource.Resource):
    base_path = '/fgs/triggers/%(function_urn)s'

    # Capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True
    allow_commit = True

    requires_id = False
    # Properties
    function_urn = resource.URI('function_urn', type=str)
    # Trigger type.
    # TIMER
    # APIG
    # CTS: Enable CTS first.
    # DDS: Configure a VPC for the function first.
    # DMS: Configure a DMS agency first.
    # DIS: Configure a DIS agency first.
    # LTS: Configure an LTS agency first.
    # OBS
    # SMN
    # KAFKA
    trigger_type_code = resource.Body(
        'trigger_type_code', type=str, alternate_id=True
    )
    # Trigger status. Options: ACTIVE and DISABLED.
    trigger_status = resource.Body('trigger_status', type=str)
    # Message code.
    event_type_code = resource.Body('event_type_code', type=str)
    # Trigger source event.
    event_data = resource.Body('event_data', type=TriggerEventData)

    # Attributes
    # Trigger ID.
    trigger_id = resource.Body('trigger_id', type=str)
    # Latest update time.
    updated_at = resource.Body('last_updated_time', type=str)
    created_at = resource.Body('created_time', type=str)

    def _delete_trigger(
            self, session, function_urn, trigger_type_code, trigger_id
    ):
        """Delete Trigger
        """
        urn = function_urn.rpartition(":")[0]
        url = (self.base_path % {'function_urn': urn}
               + f'/{trigger_type_code}/{trigger_id}')
        response = session.delete(url)
        exceptions.raise_from_response(response)
        return None

    def _delete_triggers(self, session, function_urn):
        """Delete all Triggers
        """
        url = self.base_path % {'function_urn': function_urn}
        response = session.delete(url)
        exceptions.raise_from_response(response)
        return None



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid
from datetime import datetime

import mock

from openstackclient.tests.unit import utils

from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.smn.v2 import topic
from otcextensions.sdk.smn.v2 import template
from otcextensions.sdk.smn.v2 import subscription


def gen_data(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(getattr(data, attr, '') for attr in columns)


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(data.get(attr, '') for attr in columns)


class TestSmn(utils.TestCommand):
    def setUp(self):
        super(TestSmn, self).setUp()

        self.app.client_manager.smn = mock.Mock()

        self.client = self.app.client_manager.smn


class FakeTopic(test_base.Fake):
    """Fake one or more Topics."""
    @classmethod
    def generate(cls):
        """Create a fake Topic..

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            "push_policy": 0,
            "create_time": datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"),
            "update_time": datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"),
            "name": "name-" + uuid.uuid4().hex,
            "topic_urn": "id-" + uuid.uuid4().hex,
            "display_name": "display-name-" + uuid.uuid4().hex,
            "request_id": "request-" + uuid.uuid4().hex
        }

        return topic.Topic(**object_info)


class FakeTemplate(test_base.Fake):
    """Fake one or more Templates."""
    @classmethod
    def generate(cls):
        """Create a fake Templates.

        :return:
            A FakeResource object, with id, status and so on
        """
        # Set default attributes.
        object_info = {
            "message_template_id": "id-" + uuid.uuid4().hex,
            "message_template_name": "name-" + uuid.uuid4().hex,
            "protocol": "https",
            "create_time": datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"),
            "update_time": datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"),
            "request_id": "request-" + uuid.uuid4().hex,
            "tag_names": [
                "topic_id_id4"
            ],
            "content": "Test Content"
        }

        return template.Template.existing(**object_info)


class FakeSubscription(test_base.Fake):
    """Fake one or more subscriptions"""
    @classmethod
    def generate(cls):
        """Create a fake subscription.

        :return:
            A FakeResource object, with id, status and so on
        """
        # Set default attributes.
        object_info = {
            "topic_urn": "topic-id" + uuid.uuid4().hex,
            "protocol": "sms",
            "subscription_urn": "id-" + uuid.uuid4().hex,
            "owner": "owner-" + uuid.uuid4().hex,
            "endpoint": "xxxxxxxxxx",
            "remark": "",
            "status": 0
        }

        obj = subscription.Subscription.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import gateway

EXAMPLE = {
    'description': 'Test API Gateway',
    'maintain_begin': '22:00',
    'maintain_end': '02:00',
    'instance_name': 'test-gateway',
    'security_group_id': 'sg-12345',
    'vpcep_service_name': 'vpcep-test-service'
}


class TestGateway(base.TestCase):

    def test_basic(self):
        sot = gateway.Gateway()
        self.assertEqual('/apigw/instances', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = gateway.Gateway(**EXAMPLE)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['maintain_begin'], sot.maintain_begin)
        self.assertEqual(EXAMPLE['maintain_end'], sot.maintain_end)
        self.assertEqual(EXAMPLE['instance_name'], sot.instance_name)
        self.assertEqual(EXAMPLE['security_group_id'], sot.security_group_id)
        self.assertEqual(EXAMPLE['vpcep_service_name'], sot.vpcep_service_name)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Modify VPC Channel Healthcheck
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
attrs = {
    "http_code": "200",
    "path": "/vpc/demo",
    "port": 22,
    "method": "GET",
    "protocol": "http",
    "threshold_abnormal": 5,
    "threshold_normal": 2,
    "time_interval": 10,
    "timeout": 5,
    "enable_client_ssl": False
}
updated = conn.apig.modify_vpc_channel_healthcheck(
    gateway="gateway_id",
    vpc_channel="vpc_channel_id",
    **attrs
)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Subscription(resource.Resource):
    resources_key = 'subscriptions'
    base_path = '/notifications/topics/%(topic_urn)s/subscriptions'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'offset', 'limit')

    #: Resource identifier of a subscription, which is unique
    id = resource.Body('subscription_urn', alternate_id=True)
    #: Message receiving endpoint
    endpoint = resource.Body('endpoint')
    #: Subscription protocol
    #: Following protocols are supported:
    #: email, sms, http and https
    protocol = resource.Body('protocol')
    #: Project ID of the topic creator
    owner = resource.Body('owner')
    #: Remarks
    remark = resource.Body('remark')
    #: Subscription status
    #:  0: unconfirmed
    #:  1: confirmed
    #:  3: canceled
    status = resource.Body('status')
    #: Resource identifier of a topic, which is unique
    topic_urn = resource.URI('topic_urn')

    def delete(self, session, error_message=None, **kwargs):
        self.base_path = '/notifications/subscriptions'
        return super(Subscription, self).delete(
            session,
            error_message=error_message,
            **kwargs)

    @classmethod
    def list(cls, session, paginated=True, base_path=None,
             allow_unknown_params=False, **params):
        if not params.get('topic_urn'):
            base_path = '/notifications/subscriptions'
        return super(Subscription, cls).list(
            session=session,
            paginated=paginated,
            base_path=base_path,
            allow_unknown_params=allow_unknown_params,
            **params)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ApiEnvironmentVar(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/env-variables'

    allow_list = True
    allow_fetch = True
    allow_create = True
    allow_commit = True
    allow_delete = True

    resources_key = 'variables'

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'group_id',
        'env_id', 'variable_name', 'precise_search'
    )

    gateway_id = resource.URI('gateway_id')

    # The variable value can contain 1 to 255 characters.
    # Only letters, digits, and special characters (_-/.:) are allowed.
    variable_value = resource.Body('variable_value')
    # Environment ID.
    env_id = resource.Body('env_id')
    # API group ID.
    group_id = resource.Body('group_id')
    # Variable name, which can contain 3 to 32 characters,
    # starting with a letter.
    # Only letters, digits, hyphens (-), and underscores (_) are allowed.
    variable_name = resource.Body('variable_name')
    # Environment variable ID.
    id = resource.Body('id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.sdk.ces.v1 import alarm as _alarm
from otcextensions.sdk.ces.v1 import event_data as _event_data
from otcextensions.sdk.ces.v1 import metric as _metric
from otcextensions.sdk.ces.v1 import metric_data as _metric_data
from otcextensions.sdk.ces.v1 import quota as _quota


class Proxy(proxy.Proxy):

    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return super()._extract_name(
            url.lower(), service_type=service_type, project_id=project_id)

    # ======== Alarms ========
    def alarms(self, **query):
        """Return a generator of alarms

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
        :returns: A generator of alarm objects
        :rtype: :class:`~otcextensions.sdk.ces.v1.alarm.Alarm`
        """
        return self._list(_alarm.Alarm, **query)

    def get_alarm(self, alarm):
        """Return a single alarm

        :param alarm: The value can be the ID of a alarm or a
            :class:`~otcextensions.sdk.ces.v1.alarm.Alarm` instance.
        :returns: A generator of alarm objects
        :rtype: :class:`~otcextensions.sdk.ces.v1.alarm.Alarm`
        """
        return self._get(_alarm.Alarm, alarm)

    def create_alarm(self, **attrs):
        """Create a new Alarm from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`otcextensions.sdk.ces.v1.alarm.Alarm`
        """
        return self._create(_alarm.Alarm, **attrs)

    def delete_alarm(self, alarm, ignore_missing=True):
        """Delete a Alarm

        :param alarm: key id or an instance of
            :class:`otcextensions.sdk.ces.v1.alarm.Alarm`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the alarm does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent alarm.
        :returns: Alarm been deleted
        :rtype: :class:`otcextensions.sdk.ces.v1.alarm.Alarm`
        """
        return self._delete(_alarm.Alarm, alarm,
                            ignore_missing=ignore_missing)

    def find_alarm(self, name_or_id, ignore_missing=True):
        """Find a single alarm

        :param name_or_id: The name or ID of a alarm
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the alarm does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent alarm.

        :returns: ``None``
        """
        return self._find(_alarm.Alarm, name_or_id,
                          ignore_missing=ignore_missing)

    def switch_alarm_state(self, alarm):
        """Enables or disables Alarm

        :param alarm: The value can be the ID of an alarm
             or a :class:`~otcextensions.sdk.ces.v1.alarm.Alarm` instance.
        :returns: None
        """
        alarm = self._get_resource(_alarm.Alarm, alarm)
        alarm.change_alarm_status(self)

    # ======== Event-Data ========
    def event_data(self, **query):
        """Return a generator of host configurations for a specified
           event type in a specified period of time.

        :param kwargs query: Optional query parameters to be sent to limit
                              the resources being returned.
        :returns: A generator of event data objects
        :rtype: :class:`~otcextensions.sdk.ces.v1.event_data.EventData`
        """
        return self._list(_event_data.EventData, **query)

    # ======== Metrics ========
    def metrics(self, **query):
        """Return a generator of metrics

        :param kwargs query: Optional query parameters to be sent to limit
                              the resources being returned.
        :returns: A generator of metric objects
        :rtype: :class:`~otcextensions.sdk.ces.v1.metric.Metric`
        """
        return self._list(_metric.Metric, **query)

    # ======== Metric-Data ========
    def metric_data(self, **query):
        """Return a generator of Metric Data

        :param kwargs query: Optional query parameters to be sent to limit
                              the resources being returned.
        :returns: A generator of metric data objects
        :rtype: :class:`~otcextensions.sdk.ces.v1.metric_data.MetricData`
        """
        return self._list(_metric_data.MetricData, **query)

    # skipped due to lag of compliant API (resource is list not JSON)
    '''
    def create_metric_data(self, **attrs):
       """Create a new Alarm from attributes

       :param dict attrs: Keyword arguments which will be used to create
           a :class:`~otcextensions.sdk.ces.v1.metric_data.MetricData`
       """
       return self._create(_metric_data.MetricData, **attrs)
    '''

    # ======== Quotas ========
    def quotas(self):
        """Return a generator of quotas

        :returns: A generator of quota objects
        :rtype: :class:`~otcextensions.sdk.ces.v1.quota.Quota`
        """
        return self._list(_quota.Quota)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Dnat(resource.Resource):
    resources_key = 'dnat_rules'
    resource_key = 'dnat_rule'
    base_path = '/dnat_rules'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'admin_state_up', 'created_at', 'external_service_port', 'id',
        'floating_ip_address', 'floating_ip_id', 'internal_service_port',
        'limit', 'nat_gateway_id', 'port_id', 'private_ip', 'protocol',
        'status', 'project_id', project_id='tenant_id'
    )

    # Properties
    #: Specifies whether DNAT rule is enabled / disabled
    #: *true:* DNAT rule is enabled
    #: *false:* DNAT rule is disabled
    admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies when the rule is created.
    #: The format is yyyy-mm-dd hh:mm:ss.
    created_at = resource.Body('created_at')
    #: Specifies the port for providing external services.
    external_service_port = resource.Body('external_service_port', type=int)
    #: Specifies the EIP
    floating_ip_address = resource.Body('floating_ip_address')
    #: Specifies the EIP ID
    floating_ip_id = resource.Body('floating_ip_id')
    #: Specifies the gateway ID.
    gateway_id = resource.Body('gateway_id')
    #: Specifies the ID of the DNAT rule.
    id = resource.Body('id')
    #: Specifies port used by ECS/BMS to provide services for external systems
    internal_service_port = resource.Body('internal_service_port', type=int)
    #: Specifies the ID of the NAT gateway.
    nat_gateway_id = resource.Body('nat_gateway_id')
    #: Specifies the port ID of an ECS or BMS
    #: Parameter is used in the VPC scenario.
    #: This parameter is an alternative to private_ip
    port_id = resource.Body('port_id')
    #: Specifies the IP address of a Direct Connect connection.
    #: Parameter is used in the Direct Connect scenario.
    #: This parameter is an alternative to port_id.
    private_ip = resource.Body('private_ip')
    #: Specifies the project ID.
    project_id = resource.Body('tenant_id')
    #: Specifies the protocol type. Currently TCP(6), UDP(17) and ANY(0)
    protocol = resource.Body('protocol')
    #: Specifies the status of the DNAT rule
    status = resource.Body('status')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import utils
from openstack import exceptions


class Attachment(resource.Resource):
    #: Properties
    #: Protected instance attached disk name
    device = resource.Body('device')
    #: Protected instance ID
    protected_instance = resource.Body('protected_instance')


class Metadata(resource.Resource):
    #: Properties
    #: Specifies whether disk is system
    bootable = resource.Body('bootable', type=bool)
    #: Specifies whether disk is shared
    multiattach = resource.Body('multiattach', type=bool)
    #: Replication pair disk size
    volume_size = resource.Body('volume_size', type=int)
    #: Replication pair disk type
    volume_type = resource.Body('volume_type')


class ReplicationPair(resource.Resource):
    """SDRS Replication pair Resource"""
    resource_key = 'replication'
    resources_key = 'replications'
    base_path = '/replications'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'availability_zone', 'limit', 'marker', 'name', 'offset',
        'protected_instance_id', 'protected_instance_ids', 'query_type',
        'server_group_id', 'server_group_ids', 'status')

    #: Properties
    #: Attached replication pair info
    attachment = resource.Body('attachment', type=list, list_type=Attachment)
    #: Creation time
    created_at = resource.Body('created_at')
    #: Option to delete DR site disk
    delete_target_volume = resource.Body('delete_target_volume',
                                         type=bool)
    #: Replication pair description
    description = resource.Body('description')
    #: Replication pair error code
    failure_detail = resource.Body('failure_detail')
    #: Replication pair fault level
    fault_level = resource.Body('fault_level')
    #: Job id for created task
    job_id = resource.Body('job_id')
    #: Replication pair ID
    id = resource.Body('id')
    #: Replication pair name
    name = resource.Body('name')
    #: Production site AZ of the protection group
    #: containing the replication pair
    priority_station = resource.Body('priority_station')
    #: Replication pair synchronization progress
    progress = resource.Body('progress', type=int)
    #: Replication pair SDR data
    record_metadata = resource.Body('record_metadata', type=Metadata)
    #: Replication pair mode
    #: Default: 'hypermetro'
    replication_model = resource.Body('replication_model')
    #: Synchronization status
    replication_status = resource.Body('replication_status')
    #: Protection group ID
    server_group_id = resource.Body('server_group_id')
    #: Replication pair status
    status = resource.Body('status')
    #: Update time
    updated_at = resource.Body('updated_at')
    #: Production site volume ID
    volume_id = resource.Body('volume_id')
    #: Production and DR site disk IDs
    volume_ids = resource.Body('volume_ids')

    def delete(self, session,
               server_group_id=None, delete_target_volume=False,
               ignore_missing=True):
        """Delete the remote resource based on this instance.

        This function overrides default Resource.delete to enable params

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool server_group_id: Specifies the ID of protection
            group
        :param bool delete_target_volume: Specifies whether DR site
            EVS disk should be deleted after replication pair deletion
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the replication pair does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent replication pair
        :return: This :class:`Replication` instance.
        """
        body = {
            'replication': {
                'delete_target_volume': delete_target_volume
            }
        }
        if server_group_id:
            body['replication']['server_group_id'] = server_group_id
        request = self._prepare_request()
        response = session.delete(request.url,
                                  json=body)
        try:
            self._translate_response(response, has_body=True)
        except exceptions.ResourceNotFound:
            if ignore_missing:
                return None
            raise
        return self

    def expand_replication(self, session,
                           replication, new_size):
        """Method to expand replication pair

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str replication: ID of replication pair
            which will be expanded
        :param int new_size: Replication pair new size
        """
        url = utils.urljoin(self.base_path,
                            replication,
                            'action')
        body = {
            'extend-replication': {
                'new_size': new_size
            }
        }

        return session.post(url, json=body)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.sdrs.v1 import job
from otcextensions.tests.unit.osclient.sdrs.v1 import fakes


class TestJob(fakes.TestSDRS):
    object = fakes.FakeJob.create_one()
    columns = ('id', 'status', 'job_type',
               'begin_time', 'end_time')

    data = [object.job_id,
            object.status,
            object.job_type,
            object.begin_time,
            object.end_time]

    def setUp(self):
        super(TestJob, self).setUp()

        self.cmd = job.ShowJob(self.app, None)
        self.client.get_job = mock.Mock(return_value=self.object)
        self.client.api_mock = self.client.get_job
        if not self.object.error_code:
            self.data, self.columns = job._add_sub_jobs_to_obj(
                self.object,
                self.data,
                self.columns)

        else:
            self.data, self.columns = job._add_parsed_task_to_obj(
                self.object,
                self.data,
                self.columns)

    def test_flatten(self):
        obj = fakes.FakeJob.create_one()

        flat_data = job._flatten_job(obj)

        data = (
            flat_data['id'],
            flat_data['status'],
            flat_data['job_type'],
            flat_data['begin_time'],
            flat_data['end_time']
        )

        cmp_data = (
            obj.job_id,
            obj.status,
            obj.job_type,
            obj.begin_time,
            obj.end_time
        )

        self.assertEqual(data, cmp_data)

    def test_default(self):
        arglist = [
            'job'
        ]

        verifylist = [('job', 'job')]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_job.assert_called_once_with(job='job')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
# from openstack import exceptions
from openstack import resource

from otcextensions.sdk import sdk_resource

_logger = _log.setup_logging('openstack')


class Flavor(sdk_resource.Resource):

    base_path = '/flavors'
    # In difference to regular OS services RDS
    # does not return single item with a proper element tag
    resource_key = 'flavor'
    resources_key = 'flavors'

    # capabilities
    allow_get = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'dbId', 'region'
    )

    # Properties
    #: Flavor id
    id = None
    #: Flavor name
    name = resource.Body('name')
    #: Ram size in MB.
    #: *Type:int*
    ram = resource.Body('ram', type=int)
    #: Specification code
    #: *Type: str*
    spec_code = resource.Body('specCode')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import utils


class VolumeSpec(resource.Resource):
    #: Properties
    #: Volume id
    id = resource.Body('id')
    #: OS type
    os_version = resource.Body('os_version')


class ResourceExtraInfoSpec(resource.Resource):
    #: Properties
    #: ID of the disk that is excluded from the backup
    exclude_volumes = resource.Body('exclude_volumes', type=list)
    #: Disk to be backed up
    include_volumes = resource.Body('include_volumes', type=list,
                                    list_type=VolumeSpec)


class ResourceSpec(resource.Resource):
    #: Properties
    #: Number of backups
    backup_count = resource.Body('backup_count', type=int)
    #: Backup size
    backup_size = resource.Body('backup_size', type=int)
    #: Extra info of the resource
    extra_info = resource.Body('extra_info', type=ResourceExtraInfoSpec)
    #: ID of the resource to be backed up
    id = resource.Body('id')
    #: Resource name
    name = resource.Body('name')
    #: Protection status
    #: values: available, error, protecting, restoring, removing
    protect_status = resource.Body('protect_status')
    #: Allocated capacity for the associated resource, in GB
    size = resource.Body('size', type=int)
    #: type of the resource to be backed up.
    #: values: OS::Nova::Server, OS::Cinder::Volume
    type = resource.Body('type')


class BillingExtraInfoSpec(resource.Resource):
    #: Properties
    #: Number of items in the aplication for creating vaults in the
    #: combination mode
    combined_order_ecs_num = resource.Body('combined_order_ecs_num')
    #: ID of the application for creating vaults in combination
    combined_order_id = resource.Body('combined_order_id')


class BillingSpec(resource.Resource):
    #: Properties
    #: Allocated capacity in MB
    allocated = resource.Body('allocated', type=int)
    #: Biling mode
    #: values: post_paid, pre_paid
    #: default: post_paid
    charging_mode = resource.Body('charging_mode')
    #: Cloud type
    #: values: public, hybrid
    cloud_type = resource.Body('cloud_type')
    #: Backup specifications
    #: default: crash_consistent
    consistent_level = resource.Body('consistent_level')
    #: Redirection URL
    console_url = resource.Body('console_url')
    #: Billing extra info spec
    extra_info = resource.Body('extra_info', type=BillingExtraInfoSpec)
    #: Scenario when an account is frozen
    frozen_scene = resource.Body('frozen_scene')
    #: Whether the fee is automatically deducted from the customers account
    #: default: false
    is_auto_pay = resource.Body('is_auto_pay', type=bool)
    #: Whether to automatically renew the subscirption after expiration
    #: default: false
    is_auto_renew = resource.Body('is_auto_renew', type=bool)
    #: Object type
    object_type = resource.Body('object_type')
    #: Order ID
    order_id = resource.Body('order_id')
    #: Required duration for the package
    #: mandatory if charging_mode is set to pre_paid
    period_num = resource.Body('period_num', type=int)
    #: Package type
    #: mandatory if charging_mode is pre_paid
    #: values: year, month
    period_type = resource.Body('period_type')
    #: Product ID
    product_id = resource.Body('product_id')
    #: Operation type
    protect_type = resource.Body('operation_type')
    #: Capicity in GB
    #: min: 1, max: 10485760
    size = resource.Body('size', type=int)
    #: Specification code
    spec_code = resource.Body('spec_code')
    #: Status
    #: values: available, lock, frozen, deleting, error
    status = resource.Body('status')
    #: Name of the bucket of the vault
    storage_unit = resource.Body('storage_unit')
    #: Used capacity in MB
    used = resource.Body('used', type=int)


class Vault(resource.Resource):
    """CBR Vault Resource"""
    resource_key = 'vault'
    resources_key = 'vaults'
    base_path = '/vaults'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'cloud_type', 'id', 'limit', 'name', 'object_type', 'offset',
        'policy_id', 'protect_type', 'resources_ids', 'status')

    #: Properties
    auto_bind = resource.Body('auto_bind', type=bool)
    #: Whether to automatically expand the vault capacity
    auto_expand = resource.Body('auto_expand', type=bool)
    #: Backup Policy ID
    backup_policy_id = resource.Body('backup_policy_id')
    #: Billing spec
    billing = resource.Body('billing', type=BillingSpec)
    #: Rules for automatic association
    bind_rules = resource.Body('bind_rules', type=dict)
    #: Creation time
    created_at = resource.Body('created_at')
    #: Description
    description = resource.Body('description')
    #: Vault name
    name = resource.Body('name')
    #: Project ID
    project_id = resource.Body('project_id')
    #: Vault type
    provider_id = resource.Body('provider_id')
    #: Associated Resources
    resources = resource.Body('resources', type=list, list_type=ResourceSpec)
    #: Tag list up to 10 key value pairs
    tags = resource.Body('tags', type=list, list_type=dict)
    #: User ID
    user_id = resource.Body('user_id')

    def bind_policy(self, session, policy_id):
        """Method to associate a CBR policy to a CBR vault

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str policy_id: The ID of the policy being attached to the
            CBR vault.
        """
        url = utils.urljoin(self.base_path, self.id, 'associatepolicy')
        body = {
            'policy_id': policy_id
        }
        return session.post(url, json=body)

    def unbind_policy(self, session, policy_id):
        """Method to dissociate a CBR policy from a CBR vault

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str policy_id: The ID of the policy being attached to the
            CBR vault.
        """
        url = utils.urljoin(self.base_path, self.id, 'dissociatepolicy')
        body = {
            'policy_id': policy_id
        }
        return session.post(url, json=body)

    def associate_resources(self, session, resources):
        """Method to bind one or more ECS or Volume to a CBR vault

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param resources: array of resources in the format: name, type
        """
        url = utils.urljoin(self.base_path, self.id, 'addresources')
        body = {
            'resources': resources
        }
        return session.post(url, json=body)

    def dissociate_resources(self, session, resources):
        """Method to release one or more ECS or Volume to a CBR vault

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param resources: list of ressource ids to be released from vault
        """
        url = utils.urljoin(self.base_path, self.id, 'removeresources')
        body = {
            'resource_ids': resources
        }
        return session.post(url, json=body)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create an Instance
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    'name': 'test-instance-2',
    'engine': 'kafka',
    'engine_version': '2.3.0',
    'storage_space': '600',
    'vpc_id': 'vpc_id',
    'security_group_id': 'sec_group_id',
    'subnet_id': 'network_id',
    'available_zones': ['az_id'],
    'product_id': 'product_id',
    'storage_spec_code': 'dms.physical.storage.ultra'
}
for raw in conn.dms.create_instance(**attrs):
    print(raw)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from cliff import columns as cliff_columns

DATASET_TYPES_VALUE_MAP = {
    0: "image classification",
    1: "object detection",
    100: "text classification",
    101: "named entity recognition",
    102: "text triplet",
    200: "sound classification",
    201: "speech content",
    202: "speech paragraph labeling",
    400: "table dataset",
    600: "video labeling",
    900: "custom format",
}

DATASET_STATUS_VALUE_MAP = {
    0: "creating dataset",
    1: "normal dataset",
    2: "deleting dataset",
    3: "deleted dataset",
    4: "abnormal dataset",
    5: "synchronizing dataset",
    6: "releasing dataset",
    7: "dataset in version switching",
    8: "importing dataset",
}

SAMPLE_TYPES_VALUE_MAP = {
    0: "Image",
    1: "Text",
    2: "Speech",
    4: "Table",
    6: "Video",
    9: "Custom_Format",
}


class DatasetType(cliff_columns.FormattableColumn):
    def human_readable(self):
        return DATASET_TYPES_VALUE_MAP.get(self._value, str(self._value))


class DatasetStatus(cliff_columns.FormattableColumn):
    def human_readable(self):
        return DATASET_STATUS_VALUE_MAP.get(self._value, str(self._value))


class SampleType(cliff_columns.FormattableColumn):
    def human_readable(self):
        return SAMPLE_TYPES_VALUE_MAP.get(self._value, str(self._value))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""VPC Endpoint Service v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class ListConnections(command.Lister):

    _description = _('List VPC Endpoint Service Connections.')
    columns = (
        'Id',
        'Domain Id',
        'Status',
        'Created At',
        'Updated At',
    )

    def get_parser(self, prog_name):
        parser = super(ListConnections, self).get_parser(prog_name)

        parser.add_argument(
            'service',
            metavar='<service>',
            help=_('ID or name of the VPC Endpoint Service.'),
        )
        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_('VPC Endpoint ID.'),
        )
        parser.add_argument(
            '--marker-id',
            metavar='<marker_id>',
            help=_('Packet ID of the VPC endpoint.'),
        )
        parser.add_argument(
            '--sort-key',
            metavar='{created_at, updated_at}',
            type=lambda s: s.lower(),
            choices=['created_at', 'updated_at'],
            help=_('Sorting field of the VPC endpoint service list.'),
        )
        parser.add_argument(
            '--sort-dir',
            metavar='{asc, desc}',
            type=lambda s: s.lower(),
            choices=['asc', 'desc'],
            help=_('Sorting order of the VPC endpoint service list.'),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit number of endpoint connections queried.'),
        )
        parser.add_argument(
            '--offset',
            metavar='<offset>',
            type=int,
            help=_('Connection records after this Offset will be queried.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep
        args_list = [
            'id',
            'marker_id',
            'limit',
            'offset',
            'sort_key',
            'sort_dir',
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        endpoint_service = client.find_service(parsed_args.service)
        data = client.service_connections(endpoint_service, **attrs)

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )


class ManageConnections(command.Lister):
    _description = _('Manage VPC Endpoint Service Connections.')

    columns = (
        'Id',
        'Domain Id',
        'Status',
        'Created At',
        'Updated At',
    )

    def get_parser(self, prog_name):
        parser = super(ManageConnections, self).get_parser(prog_name)

        parser.add_argument(
            'service',
            metavar='<service>',
            help=_('ID or name of the VPC Endpoint Service.'),
        )
        parser.add_argument(
            'endpoint',
            metavar='<endpoint>',
            nargs='+',
            help=_(
                'VPC Endpoint(s) ID to Accept Or Reject '
                'Connection to Service.'
            ),
        )
        manage_request_group = parser.add_mutually_exclusive_group(
            required=True
        )
        manage_request_group.add_argument(
            '--accept',
            action='store_true',
            dest='receive',
            help=('Accept VPC Endpoint Connection to Endpoint Service.'),
        )
        manage_request_group.add_argument(
            '--reject',
            action='store_true',
            help=('Reject VPC Endpoint Connection to Endpoint Service.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep
        set_args = ('receive', 'reject')
        request_status = [
            request for request in set_args if getattr(parsed_args, request)
        ]

        endpoint_service = client.find_service(parsed_args.service)
        data = client.manage_service_connections(
            endpoint_service,
            endpoints=parsed_args.endpoint,
            action=request_status[0],
        )

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import export_api

EXAMPLE_EXPORT = {
    'env_id': 'env-123',
    'group_id': 'group-456',
    'define': 'full',
    'type': 'json',
    'version': '1.0.0',
    'apis': ['api-1', 'api-2'],
}

EXAMPLE_IMPORT = {
    'is_create_group': True,
    'group_id': 'group-789',
    'extend_mode': 'merge',
    'simple_mode': False,
    'mock_mode': True,
    'api_mode': 'overwrite',
    'file_name': 'spec_file.json',
    'success': [{'id': 'api-1', 'method': 'GET', 'path': '/success',
                 'action': 'import'}],
    'failure': [{'method': 'POST', 'path': '/fail', 'error_code': '500',
                 'error_msg': 'Internal Error'}],
    'swagger': {'id': 'swag-1', 'result': 'valid'},
    'ignore': [{'method': 'DELETE', 'path': '/ignore'}],
}


class TestExportApi(base.TestCase):

    def test_basic(self):
        sot = export_api.ExportApi()
        self.assertEqual('apigw/instances/%(gateway_id)s/openapi/export',
                         sot.base_path)

    def test_make_it(self):
        sot = export_api.ExportApi(**EXAMPLE_EXPORT)
        self.assertEqual(EXAMPLE_EXPORT['env_id'], sot.env_id)
        self.assertEqual(EXAMPLE_EXPORT['group_id'], sot.group_id)
        self.assertEqual(EXAMPLE_EXPORT['define'], sot.define)
        self.assertEqual(EXAMPLE_EXPORT['type'], sot.type)
        self.assertEqual(EXAMPLE_EXPORT['version'], sot.version)
        self.assertEqual(EXAMPLE_EXPORT['apis'], sot.apis)


class TestImportApi(base.TestCase):

    def test_basic(self):
        sot = export_api.ImportApi()
        self.assertEqual('apigw/instances/%(gateway_id)s/openapi/import',
                         sot.base_path)

    def test_make_it(self):
        sot = export_api.ImportApi(**EXAMPLE_IMPORT)
        self.assertTrue(sot.is_create_group)
        self.assertEqual(EXAMPLE_IMPORT['group_id'], sot.group_id)
        self.assertEqual(EXAMPLE_IMPORT['extend_mode'], sot.extend_mode)
        self.assertEqual(EXAMPLE_IMPORT['simple_mode'], sot.simple_mode)
        self.assertEqual(EXAMPLE_IMPORT['mock_mode'], sot.mock_mode)
        self.assertEqual(EXAMPLE_IMPORT['api_mode'], sot.api_mode)
        self.assertEqual(EXAMPLE_IMPORT['file_name'], sot.file_name)
        self.assertIsInstance(sot.success[0], export_api.SuccessSpec)
        self.assertEqual('api-1', sot.success[0].id)
        self.assertIsInstance(sot.failure[0], export_api.FailureSpec)
        self.assertEqual('/fail', sot.failure[0].path)
        self.assertIsInstance(sot.swagger, export_api.SwaggerSpec)
        self.assertEqual('swag-1', sot.swagger.id)
        self.assertIsInstance(sot.ignore[0], export_api.IgnoreSpec)
        self.assertEqual('DELETE', sot.ignore[0].method)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from openstack import exceptions

from otcextensions.tests.unit.sdk import base


class TestCceMixin(base.TestCase):

    flavor = {'id': 1, 'name': 'test'}

    def setUp(self):
        super(TestCceMixin, self).setUp()

    def test_create_cce_cluster(self):
        attrs = {
            'name': 'cluster_name',
            'type': 'VirtualMachine',
            'flavor': 'flavor1',
            'router': 'my_router',
            'network': 'my_net',
            'annotations': {'a1': 'av1'},
            'labels': {'l1': 'lv1'},
            'authentication_proxy_ca': 'some_auth_proxy_ca',
            'authentication_mode': 'x509',
            'availability_zone': 'az1',
            'billing_mode': 'bill_mode',
            'container_network_cidr': 'cncidr',
            'container_network_mode': 'overlay_l2',
            'cpu_manager_policy': 'cmp',
            'dss_master_volumes': 'dss_m_v',
            'description': 'descr',
            'external_ip': '1.2.3.4',
            'fix_pool_mask': 'fpm',
            'service_ip_range': 'sir',
            'kube_proxy_mode': 'kpm',
            'upgrade_from': 'uf',
            'version': 'v2'
        }

        self.register_uris([
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['my_router']
                ),
                json={'id': 'router_id'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['my_net']
                ),
                json={'id': 'net_id'}
            ),
            dict(
                method='POST',
                uri=self.get_cce_url(
                    base_url_append='clusters'
                ),
                status_code=200,
                json={'metadata': {'uid': '123'}, 'status':
                      {'jobID': 'job123'}},
                validate=dict(
                    json={
                        'apiVersion': 'v3',
                        'kind': 'Cluster',
                        'metadata': {
                            'annotations': {'a1': 'av1'},
                            'labels': {'l1': 'lv1'},
                            'name': 'cluster_name',
                        },
                        'spec': {
                            'authentication': {
                                'authenticatingProxy': {
                                    'ca': 'some_auth_proxy_ca',
                                },
                                'mode': 'x509',
                            },
                            'billingMode': 'bill_mode',
                            'containerNetwork': {
                                'cidr': 'cncidr',
                                'mode': 'overlay_l2',
                            },
                            'description': 'descr',
                            'extendedParam': {
                                'clusterAZ': 'az1',
                                'dssMasterVolumes': 'dss_m_v',
                                'clusterExternalIP': '1.2.3.4',
                                'alpha.cce/fixPoolMask': 'fpm',
                                'kubernetes.io/cpuManagerPolicy': 'cmp',
                                'upgradefrom': 'uf',
                            },
                            'flavor': 'flavor1',
                            'hostNetwork': {
                                'subnet': 'net_id',
                                'vpc': 'router_id',
                            },
                            'kubeProxyMode': 'kpm',
                            'kubernetesSvcIpRange': 'sir',
                            'type': 'VirtualMachine',
                            'version': 'v2'
                        }
                    }
                ),
            ),
            dict(
                method='GET',
                uri=self.get_cce_url(
                    base_url_append='jobs',
                    append=['job123']
                ),
                json={
                    'metadata': {'uid': 'job123'},
                    'status': {'phase': 'Success'},
                },
            ),
            dict(
                method='GET',
                uri=self.get_cce_url(
                    base_url_append='clusters/123'
                ),
                json={'metadata': {'uid': '123'}}
            ),

        ])

        obj = self.cloud.create_cce_cluster(**attrs)
        self.assert_calls()

        self.assertEqual('123', obj.id)

    def test_create_cce_cluster_no_wait(self):
        attrs = {
            'name': 'cluster_name',
            'type': 'VirtualMachine',
            'flavor': 'flavor1',
            'router': 'my_router',
            'network': 'my_net',
            'annotations': {'a1': 'av1'},
            'labels': {'l1': 'lv1'},
            'authentication_proxy_ca': 'some_auth_proxy_ca',
            'authentication_mode': 'x509',
            'availability_zone': 'az1',
            'billing_mode': 'bill_mode',
            'container_network_cidr': 'cncidr',
            'container_network_mode': 'overlay_l2',
            'cpu_manager_policy': 'cmp',
            'dss_master_volumes': 'dss_m_v',
            'description': 'descr',
            'external_ip': '1.2.3.4',
            'fix_pool_mask': 'fpm',
            'service_ip_range': 'sir',
            'kube_proxy_mode': 'kpm',
            'upgrade_from': 'uf',
            'version': 'v2'
        }

        self.register_uris([
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['my_router']
                ),
                json={'id': 'router_id'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['my_net']
                ),
                json={'id': 'net_id'}
            ),
            dict(
                method='POST',
                uri=self.get_cce_url(
                    base_url_append='clusters'
                ),
                status_code=200,
                json={'metadata': {'uid': '123'}, 'status':
                      {'jobID': 'job123'}},
                validate=dict(
                    json={
                        'apiVersion': 'v3',
                        'kind': 'Cluster',
                        'metadata': {
                            'annotations': {'a1': 'av1'},
                            'labels': {'l1': 'lv1'},
                            'name': 'cluster_name',
                        },
                        'spec': {
                            'authentication': {
                                'authenticatingProxy': {
                                    'ca': 'some_auth_proxy_ca',
                                },
                                'mode': 'x509',
                            },
                            'billingMode': 'bill_mode',
                            'containerNetwork': {
                                'cidr': 'cncidr',
                                'mode': 'overlay_l2',
                            },
                            'description': 'descr',
                            'extendedParam': {
                                'clusterAZ': 'az1',
                                'dssMasterVolumes': 'dss_m_v',
                                'clusterExternalIP': '1.2.3.4',
                                'alpha.cce/fixPoolMask': 'fpm',
                                'kubernetes.io/cpuManagerPolicy': 'cmp',
                                'upgradefrom': 'uf',
                            },
                            'flavor': 'flavor1',
                            'hostNetwork': {
                                'subnet': 'net_id',
                                'vpc': 'router_id',
                            },
                            'kubeProxyMode': 'kpm',
                            'kubernetesSvcIpRange': 'sir',
                            'type': 'VirtualMachine',
                            'version': 'v2'
                        }
                    }
                ),
            ),
        ])

        obj = self.cloud.create_cce_cluster(wait=False, **attrs)
        self.assert_calls()

        self.assertEqual('123', obj.id)

    def test_create_cce_cluster_bad_router(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['my_router']
                ),
                status_code=404,
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    qs_elements=['name=my_router']
                ),
                status_code=200,
                json={'routers': []}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['my_net']
                ),
                json={'id': 'net_id'}
            ),
        ])

        attrs = {
            'name': 'cluster_name',
            'type': 'VirtualMachine',
            'flavor': 'flavor1',
            'router': 'my_router',
            'network': 'my_net',
            'annotations': {'a1': 'av1'},
            'labels': {'l1': 'lv1'},
            'authentication_proxy_ca': 'some_auth_proxy_ca',
            'authentication_mode': 'x509',
            'az': 'az1',
            'billing_mode': 'bill_mode',
            'container_network_cidr': 'cncidr',
            'container_network_mode': 'overlay_l2',
            'cpu_manager_policy': 'cmp',
            'dss_master_volumes': 'dss_m_v',
            'description': 'descr',
            'external_ip': '1.2.3.4',
            'fix_pool_mask': 'fpm',
            'service_ip_range': 'sir',
            'kube_proxy_mode': 'kpm',
            'upgrade_from': 'uf',
            'version': 'v2'
        }

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_cce_cluster,
            **attrs
        )

    def test_create_cce_cluster_bad_net(self):
        self.register_uris([
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['my_router']
                ),
                json={'id': 'router_id'},
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    qs_elements=['name=my_net']
                ),
                status_code=200,
                json={'networks': []}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['my_net']
                ),
                status_code=404
            ),
        ])

        attrs = {
            'name': 'cluster_name',
            'type': 'VirtualMachine',
            'flavor': 'flavor1',
            'router': 'my_router',
            'network': 'my_net',
            'annotations': {'a1': 'av1'},
            'labels': {'l1': 'lv1'},
            'authentication_proxy_ca': 'some_auth_proxy_ca',
            'authentication_mode': 'x509',
            'availability_zone': 'az1',
            'billing_mode': 'bill_mode',
            'container_network_cidr': 'cncidr',
            'container_network_mode': 'overlay_l2',
            'cpu_manager_policy': 'cmp',
            'dss_master_volumes': 'dss_m_v',
            'description': 'descr',
            'external_ip': '1.2.3.4',
            'fix_pool_mask': 'fpm',
            'service_ip_range': 'sir',
            'kube_proxy_mode': 'kpm',
            'upgrade_from': 'uf',
            'version': 'v2'
        }

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_cce_cluster,
            **attrs
        )

    def test_create_cce_delete_cluster(self):
        attrs = {
            'cluster': 'cluster_name',
        }

        self.register_uris([
            dict(
                method='GET',
                uri=self.get_cce_url(
                    base_url_append='clusters/cluster_name'
                ),
                status_code=404
            ),
            dict(
                method='GET',
                uri=self.get_cce_url(
                    base_url_append='clusters'
                ),
                json={'items': [
                    {'metadata': {'uid': '123', 'name': 'cluster_name'}}
                ]}
            ),
            dict(
                method='DELETE',
                uri=self.get_cce_url(
                    base_url_append='clusters/123'
                ),
                json={'status': {'jobID': 'job123'}}
            ),
            dict(
                method='GET',
                uri=self.get_cce_url(
                    base_url_append='jobs',
                    append=['job123']
                ),
                json={
                    'metadata': {'uid': 'job123'},
                    'status': {'phase': 'Success'},
                },
            ),
        ])

        obj = self.cloud.delete_cce_cluster(**attrs)
        self.assert_calls()

        self.assertIsNone(obj)

    def test_create_cce_delete_cluster_no_wait(self):
        attrs = {
            'cluster': 'cluster_name',
        }

        self.register_uris([
            dict(
                method='GET',
                uri=self.get_cce_url(
                    base_url_append='clusters/cluster_name'
                ),
                status_code=404
            ),
            dict(
                method='GET',
                uri=self.get_cce_url(
                    base_url_append='clusters'
                ),
                json={'items': [
                    {'metadata': {'uid': '123', 'name': 'cluster_name'}}
                ]}
            ),
            dict(
                method='DELETE',
                uri=self.get_cce_url(
                    base_url_append='clusters/123'
                ),
                json={'status': {'jobID': 'job123'}}
            ),
        ])

        obj = self.cloud.delete_cce_cluster(wait=False, **attrs)
        self.assert_calls()

        self.assertIsNone(obj)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.sdk.cce.v3 import cluster
from otcextensions.sdk.cce.v3 import node_pool as nodePool
from otcextensions.osclient.cce.v2 import node_pool
from otcextensions.tests.unit.osclient.cce.v2 import fakes


class TestNodePool(fakes.TestCCE):

    def setUp(self):
        super(TestNodePool, self).setUp()

    def test_flatten(self):
        obj = fakes.FakeNodePool.create_one()

        flat_data = node_pool._flatten_node_pool(obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['flavor'],
            flat_data['os'],
            flat_data['current_node'],
            flat_data['network_id'],
            flat_data['root_volume_type'],
            flat_data['root_volume_size'],
            flat_data['data_volume_type'],
            flat_data['data_volume_size'],
            flat_data['autoscaling'],
            flat_data['min_node_count'],
            flat_data['max_node_count'],
            flat_data['scale_down_cooldown_time'],
            flat_data['priority'],

        )

        cmp_data = (
            obj.id,
            obj.name,
            obj.spec.node_template_spec.flavor,
            obj.spec.node_template_spec.os,
            obj.status.current_node,
            obj.spec.node_template_spec.node_nic_spec.primary_nic.network_id,
            obj.spec.node_template_spec.root_volume.type,
            obj.spec.node_template_spec.root_volume.size,
            obj.spec.node_template_spec.data_volumes[0].type,
            obj.spec.node_template_spec.data_volumes[0].size,
            obj.spec.autoscaling.enable,
            obj.spec.autoscaling.min_node_count,
            obj.spec.autoscaling.max_node_count,
            obj.spec.autoscaling.scale_down_cooldown_time,
            obj.spec.autoscaling.priority
        )

        self.assertEqual(data, cmp_data)


class TestListNodePool(fakes.TestCCE):

    _objs = fakes.FakeNodePool.create_multiple(3)

    columns = (
        'ID',
        'name',
        'flavor',
        'os',
        'autoscaling',
        'current_node',
        'status')

    data = []

    for s in _objs:
        flat_data = node_pool._flatten_node_pool(s)

        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['flavor'],
            flat_data['os'],
            flat_data['autoscaling'],
            flat_data['current_node'],
            flat_data['status'],
        ))

    def setUp(self):
        super(TestListNodePool, self).setUp()

        self.cmd = node_pool.ListCCENodePools(self.app, None)

        self.client.node_pools = mock.Mock()
        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_id'))

    def test_list_default(self):
        arglist = ['cluster_id']

        verifylist = [
            ('cluster', 'cluster_id')
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.node_pools.side_effect = [
            self._objs
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.node_pools.assert_called_once_with(
            'cluster_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowNodePool(fakes.TestCCE):

    _obj = fakes.FakeNodePool.create_one()

    columns = (
        'ID',
        'name',
        'flavor',
        'os',
        'current_node',
        'network_id',
        'root_volume_type',
        'root_volume_size',
        'data_volume_type',
        'data_volume_size',
        'autoscaling',
        'min_node_count',
        'max_node_count',
        'scale_down_cooldown_time',
        'priority',
        'status',)

    flat_data = node_pool._flatten_node_pool(_obj)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['flavor'],
        flat_data['os'],
        flat_data['current_node'],
        flat_data['network_id'],
        flat_data['root_volume_type'],
        flat_data['root_volume_size'],
        flat_data['data_volume_type'],
        flat_data['data_volume_size'],
        flat_data['autoscaling'],
        flat_data['min_node_count'],
        flat_data['max_node_count'],
        flat_data['scale_down_cooldown_time'],
        flat_data['priority'],
        flat_data['status']
    )

    def setUp(self):
        super(TestShowNodePool, self).setUp()

        self.cmd = node_pool.ShowCCENodePool(self.app, None)

        self.client.find_node_pool = mock.Mock()

        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_uuid'))

    def test_show(self):
        arglist = [
            'cluster_uuid',
            'pool_id'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid'),
            ('nodepool', 'pool_id')
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_node_pool.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_node_pool.assert_called_once_with(
            cluster='cluster_uuid',
            node_pool='pool_id')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteNodePool(fakes.TestCCE):

    def setUp(self):
        super(TestDeleteNodePool, self).setUp()

        self.cmd = node_pool.DeleteCCENodePool(self.app, None)

        self.client.delete_node_pool = mock.Mock()

        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_uuid'))

        self.client.find_node_pool = mock.Mock()

    def test_delete(self):
        arglist = [
            'cluster_uuid',
            'pool1',
            'pool2'
        ]

        verifylist = [
            ('cluster', 'cluster_uuid'),
            ('nodepool', ['pool1', 'pool2'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_node_pool.side_effect = [{}, {}]

        # Set the response for find_cluster
        self.client.find_node_pool.side_effect = [
            nodePool.NodePool(id='pool1'),
            nodePool.NodePool(id='pool2')]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        delete_calls = [
            mock.call(cluster='cluster_uuid', node_pool='pool1',
                      ignore_missing=False),
            mock.call(cluster='cluster_uuid', node_pool='pool2',
                      ignore_missing=False)
        ]

        find_calls = [
            mock.call(cluster='cluster_uuid', node_pool='pool1'),
            mock.call(cluster='cluster_uuid', node_pool='pool2')
        ]

        self.client.delete_node_pool.assert_has_calls(delete_calls)
        self.client.find_node_pool.assert_has_calls(find_calls)
        self.assertEqual(2, self.client.delete_node_pool.call_count)


class TestCreateNodePool(fakes.TestCCE):

    _obj = fakes.FakeNodePool.create_one()

    columns = (
        'ID',
        'name',
        'flavor',
        'os',
        'current_node',
        'network_id',
        'root_volume_type',
        'root_volume_size',
        'data_volume_type',
        'data_volume_size',
        'autoscaling',
        'min_node_count',
        'max_node_count',
        'scale_down_cooldown_time',
        'priority',)

    flat_data = node_pool._flatten_node_pool(_obj)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['flavor'],
        flat_data['os'],
        flat_data['current_node'],
        flat_data['network_id'],
        flat_data['root_volume_type'],
        flat_data['root_volume_size'],
        flat_data['data_volume_type'],
        flat_data['data_volume_size'],
        flat_data['autoscaling'],
        flat_data['min_node_count'],
        flat_data['max_node_count'],
        flat_data['scale_down_cooldown_time'],
        flat_data['priority'],
    )

    def setUp(self):
        super(TestCreateNodePool, self).setUp()

        self.cmd = node_pool.CreateCCENodePool(self.app, None)

        self.app.client_manager.sdk_connection = mock.Mock()

        self.cloud_client = self.app.client_manager.sdk_connection

        self.cloud_client.create_cce_node_pool = mock.Mock()

        self.client.find_cluster = mock.Mock(
            return_value=cluster.Cluster(id='cluster_id'))

    def test_create(self):
        arglist = [
            'cluster_name',
            'pool_name',
            '--flavor', 'flav',
            '--network', 'nw',
            '--os', 'CentOS',
            '--ssh-key', 'sshkey'
        ]

        verifylist = [
            ('cluster', 'cluster_name'),
            ('name', 'pool_name'),
            ('autoscaling_enabled', False),
            ('az', 'random'),
            ('flavor', 'flav'),
            ('initial_node_count', 0),
            ('network', 'nw'),
            ('os', 'CentOS'),
            ('root_volume_size', 40),
            ('root_volume_type', 'SATA'),
            ('ssh_key', 'sshkey'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.cloud_client.create_cce_node_pool.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.cloud_client.create_cce_node_pool.assert_called_once_with(
            availability_zone='random',
            cluster='cluster_name',
            flavor='flav',
            name='pool_name',
            network='nw',
            os='CentOS',
            root_volume_size=40,
            root_volume_type='SATA',
            ssh_key='sshkey'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import uuid

import fixtures

from openstackclient.tests.functional import base


CREATE_COMMAND = 'vbs policy create -f json %(name)s --start_time "10:00" ' \
    '--frequency 14 --rentention_num 2 --enable'

DELETE_COMMAND = 'vbs policy delete %(id)s'


class VolumeBackupPolicyTests(base.TestCase):
    """Functional tests for vbs. """

    NAME = uuid.uuid4().hex
    OTHER_NAME = uuid.uuid4().hex

    def tearDown(self):
        try:
            self.openstack(
                DELETE_COMMAND % {'id': self.policy_id}
            )
        finally:
            super(VolumeBackupPolicyTests, self).tearDown()

    def setUp(self):
        super(VolumeBackupPolicyTests, self).setUp()
        json_output = json.loads(self.openstack(
            CREATE_COMMAND % {'name': self.NAME}
        ))
        self.policy_id = json_output["id"]
        self.assertOutput(self.NAME, json_output['name'])
        ver_fixture = fixtures.EnvironmentVariable(
            'OS_VBS_API_VERSION', '2'
        )
        self.useFixture(ver_fixture)

    def test_policy_list(self):
        json_output = json.loads(self.openstack(
            'vbs policy list -f json '
        ))
        self.assertIn(
            self.NAME,
            [img['name'] for img in json_output]
        )

    def test_policy_set_rename(self):
        name = uuid.uuid4().hex
        json_output = json.loads(self.openstack(
            CREATE_COMMAND % {'name': name}
        ))
        policy_id = json_output["id"]
        self.assertEqual(
            name,
            json_output["name"],
        )
        self.openstack(
            'vbs policy update --name {name}xx {policy}'.format(
                name=name, policy=policy_id)
        )
        json_output = json.loads(self.openstack(
            'vbs policy show {name}xx -f json'.format(name=name)
        ))
        self.assertEqual(
            name + 'xx',
            json_output["name"],
        )
        self.openstack(
            DELETE_COMMAND % {'id': json_output['id']}
        )

    def test_policy_execute(self):
        self.openstack(
            'vbs policy execute ' + self.policy_id
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import json
import uuid

from openstackclient.tests.functional import base


class DcaasTestCase(base.TestCase):
    """Common functional test bits for DCAAS commands"""

    UUID = uuid.uuid4().hex[:9]
    NETWORK_NAME = "test-dc-network-" + UUID
    SUBNET_NAME = "test-dc-subnet-" + UUID
    ROUTER_NAME = "test-dc-router-" + UUID
    IP_VERSION = 4
    CIDR = "192.168.0.0/16"
    EP_GROUP_NAME = "test-dc-eg-" + UUID
    EP_GROUP_TYPE = "cidr"

    def setUp(self):
        super(DcaasTestCase, self).setUp()
        self.create_test_infra()
        self.EP_GROUP_ID = self.create_endpoint_group()

    def tearDown(self):
        try:
            self.delete_endpoint_group()
        finally:
            self.delete_test_infra()
            super(DcaasTestCase, self).tearDown()

    def _get_project_id(self):
        json_output = json.loads(self.openstack(
            'token issue -f json'
        ))
        return json_output['project_id']

    def _create_network(self):
        json.loads(self.openstack(
            'network create -f json ' + self.NETWORK_NAME
        ))

    def _delete_network(self):
        self.openstack('network delete ' + self.NETWORK_NAME)

    def _create_subnet(self):
        json_output = json.loads(self.openstack(
            'subnet create {subnet} -f json '
            '--network {network} '
            '--subnet-range {subnet_range} '.format(
                subnet=self.SUBNET_NAME,
                network=self.NETWORK_NAME,
                subnet_range=self.CIDR)
        ))
        self.SUBNET_ID = json_output['id']

    def _delete_subnet(self):
        self.openstack('subnet delete ' + self.SUBNET_ID)

    def _create_router(self):
        json_output = json.loads(self.openstack(
            'router create -f json ' + self.ROUTER_NAME
        ))
        self.openstack(
            'router add subnet {router} {subnet} '.format(
                router=self.ROUTER_NAME,
                subnet=self.SUBNET_NAME
            )
        )
        self.VPC_ID = json_output['id']

    def _router_remove_subnet(self):
        self.openstack(
            'router remove subnet {router} {subnet} '.format(
                router=self.ROUTER_NAME,
                subnet=self.SUBNET_NAME
            )
        )

    def _delete_router(self):
        self.openstack('router delete ' + self.VPC_ID)

    def create_test_infra(self):
        self._create_network()
        self._create_subnet()
        self._create_router()

    def delete_test_infra(self):
        self._router_remove_subnet()
        self._delete_subnet()
        self._delete_network()
        self._delete_router()

    def create_endpoint_group(self):
        project_id = self._get_project_id()
        endpoints = self.CIDR
        json_output = json.loads(self.openstack(
            'dcaas endpoint group create '
            '--name {name} '
            '{project_id} '
            '{type} '
            '{endpoints} -f json'.format(
                name=self.EP_GROUP_NAME,
                project_id=project_id,
                type=self.EP_GROUP_TYPE,
                endpoints=endpoints
            )
        ))
        return json_output['id']

    def delete_endpoint_group(self):
        self.openstack(
            'dcaas endpoint group delete ' + self.EP_GROUP_ID
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter
from openstack.tests.unit import base

from otcextensions.sdk.cbr.v3 import member

EXAMPLE = {
    'status': 'accepted',
    'backup_id': 'backup_id',
    'image_id': 'image_id',
    'vault_id': 'vault_id',
    'dest_project_id': 'dest_project_id',
    'updated_at': '2020-02-24T09:36:00.479031',
    'created_at': '2020-02-24T09:36:00.479033',
    'id': 'member_id'
}


class TestMember(base.TestCase):

    def setUp(self):
        super(TestMember, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = None
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.retriable_status_codes = ()
        self.sot = member.Member()

    def test_basic(self):
        sot = member.Member()
        self.assertEqual('member', sot.resource_key)
        self.assertEqual('members', sot.resources_key)
        self.assertEqual('/backups/%(backup_id)s/members', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = member.Member.existing(**EXAMPLE)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['backup_id'], sot.backup_id)
        self.assertEqual(EXAMPLE['image_id'], sot.image_id)
        self.assertEqual(EXAMPLE['vault_id'], sot.vault_id)
        self.assertEqual(EXAMPLE['dest_project_id'], sot.dest_project_id)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock
# from osc_lib import utils as common_utils

from otcextensions.osclient.rds.v1 import instance
from otcextensions.tests.unit.osclient.rds.v1 import fakes as rds_fakes


class TestListDatabaseInstances(rds_fakes.TestRds):

    column_list_headers = [
        'ID', 'Name', 'Datastore Type',
        'Datastore Version', 'Status',
        'Flavor ID', 'Type', 'Size', 'Region']

    def setUp(self):
        super(TestListDatabaseInstances, self).setUp()

        self.cmd = instance.ListDatabaseInstances(self.app, None)

        self.app.client_manager.rds.instances = mock.Mock()

        self.objects = self.instance_mock.create_multiple(3)
        self.object_data = []
        for s in self.objects:
            self.object_data.append((
                s.id,
                s.name,
                s.datastore['type'],
                s.datastore['version'],
                s.status,
                s.flavor['id'],
                s.type,
                s.volume['size'],
                s.region
                # s.flavor_detail,
            ))

    def test_list(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.instances.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.instances.assert_called()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(tuple(self.object_data), tuple(data))


class TestShowDatabaseInstance(rds_fakes.TestRds):

    column_list_headers = [
        'datastore', 'flavor', 'id', 'name', 'region'
        'status', 'volume']

    def setUp(self):
        super(TestShowDatabaseInstance, self).setUp()

        self.cmd = instance.ShowDatabaseInstance(self.app, None)

        self.app.client_manager.rds.find_instance = mock.Mock()

        self.object = self.instance_mock.create_one()

        self.object_data = (
            self.object.datastore,
            self.object.flavor,
            self.object.id,
            self.object.name,
            self.object.region,
            self.object.status,
            self.object.volume,
        )

    def test_show(self):
        arglist = [
            'test_obj',
        ]

        verifylist = [
            ('instance', 'test_obj'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.find_instance.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.find_instance.assert_called()

        # self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.object_data, data)


class TestDeleteDatabaseInstance(rds_fakes.TestRds):

    def setUp(self):
        super(TestDeleteDatabaseInstance, self).setUp()

        self.cmd = instance.DeleteDatabaseInstance(self.app, None)

        self.app.client_manager.rds.delete_instance = mock.Mock()

    def test_delete(self):
        arglist = [
            'test_obj',
        ]

        verifylist = [
            ('instance', 'test_obj'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.delete_instance.side_effect = [
            True
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.delete_instance.assert_called()


class TestResizeDatabaseInstanceFlavor(rds_fakes.TestRds):

    def setUp(self):
        super(TestResizeDatabaseInstanceFlavor, self).setUp()

        self.cmd = instance.ResizeDatabaseInstanceFlavor(self.app, None)

        self.app.client_manager.rds.find_instance = mock.Mock()
        self.app.client_manager.rds.find_flavor = mock.Mock()

        self.instance = self.instance_mock.create_one()
        self.flavor = self.flavor_mock.create_one()
        self.instance.resize = mock.Mock()

        self.instance._update(project_id='123')

    def test_delete(self):
        arglist = [
            'curr_inst',
            'new_flavor',
        ]

        verifylist = [
            ('instance', 'curr_inst'),
            ('flavor_id', 'new_flavor')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.find_instance.side_effect = [
            self.instance
        ]
        self.app.client_manager.rds.find_flavor.side_effect = [
            self.flavor
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.find_instance.assert_called()
        self.app.client_manager.rds.find_flavor.assert_called()
        self.instance.resize.assert_called()


class TestCreateDatabaseInstance(rds_fakes.TestRds):

    def setUp(self):
        super(TestCreateDatabaseInstance, self).setUp()

        self.cmd = instance.CreateDatabaseInstance(self.app, None)

        self.app.client_manager.rds.find_flavor = mock.Mock()
        self.app.client_manager.rds.create_instance = mock.Mock()

        self.instance = self.instance_mock.create_one()
        self.flavor = self.flavor_mock.create_one()
        # self.instance.resize = mock.Mock()

        # self.instance._update(project_id='123')

    def test_action(self):
        arglist = [
            '--users',
            'a:b',
            '--size',
            '15',
            'inst_name',
            'flavor',
        ]

        verifylist = [
            ('users', ['a:b']),
            ('size', 15),
            ('name', 'inst_name'),
            ('flavor', 'flavor'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.create_instance.side_effect = [
            self.instance
        ]
        self.app.client_manager.rds.find_flavor.side_effect = [
            self.flavor
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.find_flavor.assert_called()
        self.app.client_manager.rds.create_instance.assert_called()


class TestResetDatabaseInstanceStatus(rds_fakes.TestRds):

    def setUp(self):
        super(TestResetDatabaseInstanceStatus, self).setUp()

        self.cmd = instance.ResetDatabaseInstanceStatus(self.app, None)

    def test_delete(self):
        arglist = [
            'curr_inst',
        ]

        verifylist = [
            ('instance', 'curr_inst'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            NotImplementedError, self.cmd.take_action, parsed_args)


class TestUpgradeDatabaseInstance(rds_fakes.TestRds):

    def setUp(self):
        super(TestUpgradeDatabaseInstance, self).setUp()

        self.cmd = instance.UpgradeDatabaseInstance(self.app, None)

    def test_delete(self):
        arglist = [
            'curr_inst',
            'dv'
        ]

        verifylist = [
            ('instance', 'curr_inst'),
            ('datastore_version', 'dv'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            NotImplementedError, self.cmd.take_action, parsed_args)


class TestEnableDatabaseInstanceLog(rds_fakes.TestRds):

    def setUp(self):
        super(TestEnableDatabaseInstanceLog, self).setUp()

        self.cmd = instance.EnableDatabaseInstanceLog(self.app, None)

    def test_delete(self):
        arglist = [
            'curr_inst',
            'dv'
        ]

        verifylist = [
            ('instance', 'curr_inst'),
            ('log_name', 'dv'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            NotImplementedError, self.cmd.take_action, parsed_args)


class TestResizeDatabaseInstanceVolume(rds_fakes.TestRds):

    def setUp(self):
        super(TestResizeDatabaseInstanceVolume, self).setUp()

        self.cmd = instance.ResizeDatabaseInstanceVolume(self.app, None)

    def test_delete(self):
        arglist = [
            'curr_inst',
            '15'
        ]

        verifylist = [
            ('instance', 'curr_inst'),
            ('size', 15),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            NotImplementedError, self.cmd.take_action, parsed_args)


class TestForceDeleteDatabaseInstance(rds_fakes.TestRds):

    def setUp(self):
        super(TestForceDeleteDatabaseInstance, self).setUp()

        self.cmd = instance.ForceDeleteDatabaseInstance(self.app, None)

    def test_delete(self):
        arglist = [
            'curr_inst',
        ]

        verifylist = [
            ('instance', 'curr_inst'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            NotImplementedError, self.cmd.take_action, parsed_args)


class TestRestartDatabaseInstance(rds_fakes.TestRds):

    def setUp(self):
        super(TestRestartDatabaseInstance, self).setUp()

        self.cmd = instance.RestartDatabaseInstance(self.app, None)
        self.app.client_manager.rds.restart_instance = mock.Mock()

    def test_delete(self):
        arglist = [
            'curr_inst',
        ]

        verifylist = [
            ('instance', 'curr_inst'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.restart_instance.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.restart_instance.assert_called()


class TestRestoreDatabaseInstance(rds_fakes.TestRds):

    def setUp(self):
        super(TestRestoreDatabaseInstance, self).setUp()

        self.cmd = instance.RestoreDatabaseInstance(self.app, None)
        self.app.client_manager.rds.restore_instance = mock.Mock()

    def test_delete(self):
        arglist = [
            'curr_inst',
            '--backup', 'bck'
        ]

        verifylist = [
            ('instance', 'curr_inst'),
            ('backup', 'bck')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.app.client_manager.rds.restore_instance.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.app.client_manager.rds.restore_instance.assert_called()


class TestUpdateDatabaseInstance(rds_fakes.TestRds):

    def setUp(self):
        super(TestUpdateDatabaseInstance, self).setUp()

        self.cmd = instance.UpdateDatabaseInstance(self.app, None)

    def test_delete(self):
        arglist = [
            'curr_inst',
        ]

        verifylist = [
            ('instance', 'curr_inst'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            NotImplementedError, self.cmd.take_action, parsed_args)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

# import copy
import datetime
import random
import uuid

import mock
# from osc_lib import utils as common_utils

# from openstackclient.tests.unit import fakes
from openstackclient.tests.unit import utils

from otcextensions.sdk.obs.v1.bucket import Bucket
from otcextensions.sdk.obs.v1.object import Object

# from otcextensions.obs.v1.api import API


class TestObs(utils.TestCommand):

    def setUp(self):
        super(TestObs, self).setUp()

        self.app.client_manager.obs = mock.Mock()

        # s3api = API(client=self.app.client_manager.obs)
        # self.app.client_manager.obs.api = s3api

        self.bucket_mock = FakeBucket
        self.object_mock = FakeObject


class FakeBucket(object):
    """Fake one or more compute servers."""

    @staticmethod
    def create_one_bucket(attrs=None, methods=None):
        """Create a fake server.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :return:
            A FakeResource object, with id, name, metadata, and so on
        """
        attrs = attrs or {}
        methods = methods or {}

        # Set default attributes.
        object_info = {
            'creationdate': datetime.datetime(
                random.randint(2000, 2020),
                random.randint(1, 12),
                random.randint(1, 28)
            ),
            'name': 'bucket-' + uuid.uuid4().hex,
        }

        # Overwrite default attributes.
        # object_info.update(attrs)
        return Bucket(**object_info)

    @staticmethod
    def create_buckets(attrs=None, methods=None, count=2):
        """Create multiple fake servers.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :param int count:
            The number of servers to fake
        :return:
            A list of FakeResource objects faking the servers
        """
        objects = []
        for i in range(0, count):
            objects.append(FakeBucket.create_one_bucket(attrs, methods))

        return objects


class FakeObject(object):
    """Fake one or more compute servers."""

    @staticmethod
    def create_one_object(attrs=None, methods=None):
        """Create a fake server.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :return:
            A FakeResource object, with id, name, metadata, and so on
        """
        attrs = attrs or {}
        methods = methods or {}

        # Set default attributes.
        object_info = {
            'key': 'object-key-' + uuid.uuid4().hex,
            'size': random.randint(1, 99999999999),
            'lastmodified': datetime.datetime(
                random.randint(2000, 2020),
                random.randint(1, 12),
                random.randint(1, 28)
            ),
            'etag': 'image-id-' + uuid.uuid4().hex,
            'storageclass': 'storage-class-' + uuid.uuid4().hex,
        }

        # Overwrite default attributes.
        # object_info.update(attrs)
        return Object(**object_info)

    @staticmethod
    def create_objects(attrs=None, methods=None, count=2):
        """Create multiple fake servers.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :param int count:
            The number of servers to fake
        :return:
            A list of FakeResource objects faking the servers
        """
        objects = []
        for i in range(0, count):
            objects.append(FakeObject.create_one_object(attrs, methods))

        return objects



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
import uuid
from openstack import resource

from otcextensions.tests.functional import base

_logger = openstack._log.setup_logging('openstack')


class TestSnat(base.BaseFunctionalTest):

    floating_ip = None
    snat_rule = None
    uuid_v4 = uuid.uuid4().hex[:8]
    network_info = None
    gateway = None
    gateway_name = uuid_v4 + 'test-snat-gateway'
    attrs = {
        "name": gateway_name,
        "spec": "1"
    }

    def _create_network(self):
        cidr = '192.168.0.0/16'
        uuid_v4 = uuid.uuid4().hex[:8]
        vpc_name = 'snat-test-vpc-' + uuid_v4
        subnet_name = 'snat-test-subnet-' + uuid_v4

        if not TestSnat.network_info:
            vpc = self.conn.vpc.create_vpc(
                name=vpc_name,
                cidr=cidr
            )
            self.assertEqual(vpc_name, vpc.name)
            vpc_id = vpc.id
            gw, _ = cidr.split("/")
            subnet = self.conn.vpc.create_subnet(
                name=subnet_name,
                vpc_id=vpc.id,
                cidr=vpc.cidr,
                gateway_ip=gw[:-2] + ".1",
                dns_list=[
                    "100.125.4.25",
                    "100.125.129.199",
                ]
            )
            resource.wait_for_status(
                self.conn.vpc, subnet, "ACTIVE", None, 2, 20
            )
            self.assertEqual(subnet_name, subnet.name)
            subnet_id = subnet.id
            network_id = subnet.neutron_network_id

            TestSnat.network_info = {
                'vpc_id': vpc_id,
                'subnet_id': subnet_id,
                'network_id': network_id,
                'subnet': subnet,
                'vpc': vpc
            }
        if not TestSnat.gateway:
            self.attrs['router_id'] = TestSnat.network_info['vpc_id']
            self.attrs['internal_network_id'] =\
                TestSnat.network_info['network_id']
            TestSnat.gateway = self.conn.nat.create_gateway(**self.attrs)
            self.conn.nat.wait_for_gateway(TestSnat.gateway)
            self.assertIsNotNone(TestSnat.gateway)
        if not TestSnat.floating_ip:
            admin_external_net = self.conn.network.find_network(
                name_or_id='admin_external_net')
            self.assertIsNotNone(admin_external_net)
            TestSnat.floating_ip = self.conn.network.create_ip(
                floating_network_id=admin_external_net.id)

    def _destroy_network(self):
        if TestSnat.gateway:
            self.conn.nat.delete_gateway(gateway=TestSnat.gateway)
            self.conn.nat.wait_for_delete_gateway(TestSnat.gateway,
                                                  interval=4, wait=500)
            TestSnat.gateway = None
        if TestSnat.floating_ip:
            self.conn.network.delete_ip(TestSnat.floating_ip)
            TestSnat.floating_ip = None

        if TestSnat.network_info:
            vpc = TestSnat.network_info['vpc']
            subnet = TestSnat.network_info['subnet']

            resource.wait_for_status(
                self.conn.vpc, subnet, "ACTIVE", None, 2, 20
            )
            self.conn.vpc.delete_subnet(subnet, ignore_missing=False)
            resource.wait_for_delete(self.conn.vpc, subnet, 2, 60)

            sot = self.conn.vpc.delete_vpc(vpc)
            self.assertIsNotNone(sot)

            TestSnat.network_info = None

    def _create_snat_rule(self):
        TestSnat.snat_rule = self.conn.nat.create_snat_rule(
            floating_ip_id=TestSnat.floating_ip.id,
            nat_gateway_id=TestSnat.gateway.id,
            cidr='192.168.0.0/16')
        self.conn.nat.wait_for_snat(TestSnat.snat_rule)
        self.assertIsNotNone(TestSnat.snat_rule)

    def test_01_list_snat_rules(self):
        self._create_network()
        self._create_snat_rule()
        self.snat_rules = list(self.conn.nat.snat_rules())
        self.assertGreaterEqual(len(self.snat_rules), 0)

    def test_02_get_snat_rule(self):
        snat_rule = self.conn.nat.get_snat_rule(TestSnat.snat_rule.id)
        self.assertEqual(snat_rule.id, TestSnat.snat_rule.id)

    def test_03_delete_snat_rule(self):
        try:
            self.conn.nat.delete_snat_rule(snat=TestSnat.snat_rule)
            self.conn.nat.wait_for_delete_snat(TestSnat.snat_rule,
                                               interval=5, wait=250)
        except openstack.exceptions.InvalidRequest:
            self._destroy_network()
            raise
        self._destroy_network()
        try:
            snat_rule = self.conn.nat.get_snat_rule(TestSnat.snat_rule.id)
        except openstack.exceptions.ResourceNotFound:
            snat_rule = None
        self.assertIsNone(snat_rule)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DNS Zone v2 action implementations'''
import argparse
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)

ZONE_TYPES = ['private', 'public']


_formatters = {
    # 'traffic_limited_list': sdk_utils.ListOfDictColumn,
    # 'http_limited_list': sdk_utils.ListOfDictColumn,
    # 'connection_limited_list': sdk_utils.ListOfDictColumn,
}


def _get_columns(item):
    column_map = {
    }
    hidden = ['location', 'links']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


class ListZone(command.Lister):
    _description = _('List DNS zones')
    columns = (
        'id', 'name', 'zone_type', 'serial', 'status', 'action'
    )

    def get_parser(self, prog_name):
        parser = super(ListZone, self).get_parser(prog_name)
        parser.add_argument(
            '--type',
            metavar='{' + ','.join(ZONE_TYPES) + '}',
            type=lambda s: s.lower(),
            choices=ZONE_TYPES,
            help=_('Zone type. Default lists only public zones.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns

        query = {}

        if parsed_args.type:
            query['zone_type'] = parsed_args.type

        data = client.zones(**query)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns, formatters=_formatters
                 ) for s in data))
        return table


class ShowZone(command.ShowOne):
    _description = _('Show the zone details')

    def get_parser(self, prog_name):
        parser = super(ShowZone, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )

        parser.add_argument(
            '--type',
            metavar='<type>',
            choices=['private', 'public'],
            default='public',
            help=_('Mandatory for private Zone findings.\n'
                   'Choices: public, private\n'
                   'Default: public\n')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        query = {}

        if parsed_args.type and (parsed_args.type == 'private'):
            query['type'] = parsed_args.type

        obj = client.find_zone(
            parsed_args.zone,
            ignore_missing=False,
            **query
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteZone(command.Command):
    _description = _('Delete zone')

    def get_parser(self, prog_name):
        parser = super(DeleteZone, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            nargs='+',
            help=_('UUID or name of the zone.')
        )

        return parser

    def take_action(self, parsed_args):
        if parsed_args.zone:
            client = self.app.client_manager.dns
            for zone in parsed_args.zone:
                zn = client.find_zone(zone, ignore_missing=False)
                client.delete_zone(zone=zn)


class CreateZone(command.ShowOne):
    _description = _('Create zone')

    def get_parser(self, prog_name):
        parser = super(CreateZone, self).get_parser(prog_name)

        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('DNS Name for the zone.')
        )
        parser.add_argument(
            '--email',
            metavar='<email>',
            help=_('E-mail for the zone. Used in SOA records for the zone.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for this zone.')
        )
        parser.add_argument(
            '--type',
            metavar='{' + ','.join(ZONE_TYPES) + '}',
            type=lambda s: s.lower(),
            choices=ZONE_TYPES,
            help=_('Domain name type, the value of which can be '
                   '`public` or `private` .')
        )
        parser.add_argument(
            '--ttl',
            metavar='<300-2147483647>',
            type=int,
            # NOTE: py2 does not support such big int, skip unless py3-only
            # choices=range(300, 2147483647),
            help=_('TTL (Time to Live) for the zone.')
        )
        parser.add_argument(
            '--router_id',
            metavar='<router_id>',
            help=_('Router ID (VPC ID) for the private zone.')
        )
        parser.add_argument(
            '--router_region',
            metavar='<router_region>',
            help=_('Router region for the private zone.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.email:
            attrs['email'] = parsed_args.email
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.type:
            attrs['zone_type'] = parsed_args.type
        if parsed_args.ttl:
            attrs['ttl'] = parsed_args.ttl
        if parsed_args.type and parsed_args.type == 'private':
            if not parsed_args.router_id:
                msg = _('router_id is required for a private zone')
                raise argparse.ArgumentTypeError(msg)
            router = {
                'router_id': parsed_args.router_id
            }
            if parsed_args.router_region:
                router['router_region'] = parsed_args.router_region
            attrs['router'] = router

        obj = client.create_zone(
            **attrs
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class SetZone(command.ShowOne):
    _description = _('Update a Zone')

    def get_parser(self, prog_name):
        parser = super(SetZone, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )
        parser.add_argument(
            '--email',
            metavar='<email>',
            help=_('E-mail for the zone. Used in SOA records for the zone.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for this zone.')
        )
        parser.add_argument(
            '--ttl',
            metavar='<300-2147483647>',
            type=int,
            # NOTE: py2 does not support such big int, skip unless py3-only
            # choices=range(300, 2147483647),
            help=_('TTL (Time to Live) for the zone.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        attrs = {}

        if parsed_args.email:
            attrs['email'] = parsed_args.email
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.ttl:
            attrs['ttl'] = parsed_args.ttl

        zone = client.find_zone(parsed_args.zone, ignore_missing=False)

        if zone:
            obj = client.update_zone(
                zone=zone,
                **attrs
            )

            display_columns, columns = _get_columns(obj)
            data = utils.get_item_properties(obj, columns)

            return (display_columns, data)


class AssociateRouterToZone(command.ShowOne):
    _description = _('Associate router with a private zone')

    def get_parser(self, prog_name):
        parser = super(AssociateRouterToZone, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )
        parser.add_argument(
            '--router_id',
            metavar='<router_id>',
            help=_('Router ID (VPC ID) for the private zone.')
        )
        parser.add_argument(
            '--router_region',
            metavar='<router_region>',
            help=_('Router region for the private zone.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        router = {
            'router_id': parsed_args.router_id
        }
        if parsed_args.router_region:
            router['router_region'] = parsed_args.router_region

        zone = client.find_zone(parsed_args.zone, ignore_missing=False)

        if zone:
            obj = client.add_router_to_zone(
                zone=zone,
                **router
            )

            display_columns, columns = _get_columns(obj)
            data = utils.get_item_properties(obj, columns)

            return (display_columns, data)


class DisassociateRouterToZone(command.ShowOne):
    _description = _('Disassociate router with a private zone')

    def get_parser(self, prog_name):
        parser = super(DisassociateRouterToZone, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )
        parser.add_argument(
            '--router_id',
            metavar='<router_id>',
            help=_('Router ID (VPC ID) for the private zone.')
        )
        parser.add_argument(
            '--router_region',
            metavar='<router_region>',
            help=_('Router region for the private zone.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        router = {
            'router_id': parsed_args.router_id
        }
        if parsed_args.router_region:
            router['router_region'] = parsed_args.router_region

        zone = client.find_zone(parsed_args.zone, ignore_missing=False)

        if zone:
            obj = client.remove_router_from_zone(
                zone=zone,
                **router
            )

            display_columns, columns = _get_columns(obj)
            data = utils.get_item_properties(obj, columns)

            return (display_columns, data)


class ListNameserver(command.Lister):
    _description = _('List DNS zone nameservers')
    columns = (
        'address', 'hostname', 'priority'
    )

    def get_parser(self, prog_name):
        parser = super(ListNameserver, self).get_parser(prog_name)

        parser.add_argument(
            'zone',
            metavar='<zone>',
            help=_('UUID or name of the zone.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns

        zone = client.find_zone(parsed_args.zone, ignore_missing=False)

        if zone:
            data = client.nameservers(zone=zone)

            table = (self.columns,
                     (utils.get_item_properties(
                         s, self.columns, formatters=_formatters
                     ) for s in data))
            return table



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.kms.v1 import misc

EXAMPLE = {
    'random_data': 'fd7d63ce-8f5c-443e-b9a0-bef9386b23b3',
    'random_data_length': 200,
}


class TestRandom(base.TestCase):

    def setUp(self):
        super(TestRandom, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        # self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sot = misc.Random()

    def test_basic(self):
        sot = misc.Random()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual(
            '/kms/gen-random', sot.create_path)
        self.assertFalse(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        sot = misc.Random.existing(**EXAMPLE)
        self.assertEqual(EXAMPLE['random_data'], sot.random_data)
        self.assertEqual(EXAMPLE['random_data_length'], sot.random_data_length)

    def test_create(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'random_data': EXAMPLE['random_data']
        }

        self.sess.post.return_value = mock_response

        sot = misc.Random.new(random_data_length=200)
        result = sot.create(self.sess, prepend_key=False)

        call_args = self.sess.post.call_args_list[0]

        self.assertEqual('/kms/gen-random', call_args[0][0])
        self.assertDictEqual({'random_data_length': 200}, call_args[1]['json'])

        self.sess.post.assert_called_once()

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['random_data'], result.random_data)
        self.assertEqual(EXAMPLE['random_data_length'],
                         result.random_data_length)


class TestInstanceNum(base.TestCase):

    def setUp(self):
        super(TestInstanceNum, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = None
        self.sess.get = mock.Mock()
        self.sot = misc.InstanceNumber()

    def test_get(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'instance_num': 5
        }

        self.sess.get.return_value = mock_response

        sot = misc.InstanceNumber()
        result = sot.fetch(self.sess)

        self.sess.get.assert_called_once_with(
            'kms/user-instances',
            microversion=None,
            params={},
            skip_cache=False
        )

        self.assertEqual(sot, result)
        self.assertEqual(5, result.instance_num)


class TestQuota(base.TestCase):

    def setUp(self):
        super(TestQuota, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()

    def test_list(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'quotas': {
                'resources': [{
                    'type': 'CMK',
                    'used': 15,
                    'quota': 20
                }, {
                    'type': 'grant_per_CMK',
                    'used': 15,
                    'quota': 100
                }]
            }
        }

        self.sess.get.return_value = mock_response

        sot = misc.Quota()
        result = list(sot.list(self.sess))

        self.sess.get.assert_called_once_with(
            'kms/user-quotas'
        )

        self.assertIsNotNone(result)
        self.assertEqual(2, len(result))



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Import a function
"""
import openstack
from otcextensions import sdk

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)

attrs = {
    'func_name': 'test',
    'file_type': 'zip',
    'file_name': 'test.zip',
    'file_code': 'UEsDBBQAAAAIAPBbPFpOs3AMAgEAAEwCAAAGAAAAZGVwLnB5jVHB'
                 'asMwDL37K4R3iSGMMtglsNNW2HH0B4oXq9SjtY2shJbSf5/tpl5y'
                 'GFQX23qS3nuWPQZPDD/ROyGEwR3stTMHpAZHdNxC7x3jiVUnIAXT'
                 '+XbJ8QRfmiIC7xGsCwND6YHGaNaqlhVom3PwVoieD16beCNQYj6O'
                 'bGrP4wL50Ro0kNtqRch4IzfYox0nsJPtjGExboM8kAMNhDF4F7Fi'
                 '90QSdKnJHDKy5iG+e4Oyg5fVql3C396cE7CTH9kOTUI6uPxJuMra'
                 'cp0RFinFvRmOITZ3CZNiPPUYGNblsD6pjoDzr/4sawEk8hRrvjy3'
                 'D9p5/d/OOs9JNiKnxasHLSzJlfgFUEsBAhQDFAAAAAgA8Fs8Wk6z'
                 'cAwCAQAATAIAAAYAAAAAAAAAAAAAAKSBAAAAAGRlcC5weVBLBQYA'
                 'AAAAAQABADQAAAAmAQAAAAA='
}
func = conn.functiongraph.import_function(**attrs)
print(func)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create CCE Cluster
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


attrs = {
    "kind": "Cluster",
    "apiVersion": "v3",
    "metadata": {
        "name": "turbo-cluster",
    },
    "spec": {
        "type": "VirtualMachine",
        "flavor": "cce.s1.small",
        "version": "v1.19.10-r0",
        "az": "eu-de-01",
        "hostNetwork": {
            "vpc": "05c33818-78df-4329-b546-5df2f1aa823e",
            "subnet": "9f0c8ef2-c608-4238-88be-87bce5fe90da",
        },
        "containerNetwork": {
            "mode": "eni",
        },
        "eniNetwork": {
            "eniSubnetId": "417dcc1f-95d7-43e7-8533-ab078d266303",
            "eniSubnetCIDR": "10.0.0.0/14",
        },
        "authentication": {
            "mode": "rbac",
            "authenticatingProxy": {}
        },
        "billingMode": 0,
        "kubernetesSvcIpRange": "10.247.0.0/16",
        "kubeProxyMode": "iptables"
    }
}

conn.cce.create_cluster(**attrs)
